# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : RPFs.py


import sys
import pandas as pd
import polars as pl


def read_rpf_file(rpf_file):
    '''
    @Message  : import the rpf table.
    @Input    : rpf_file --> rpf table file generated from rpf_density.py
    @Return   : raw_rpf --> import rpf table with polars
    '''
    
    raw_rpf = pl.read_csv(rpf_file, has_header = True, separator = '\t')
    raw_rpf = raw_rpf.with_columns(raw_rpf['codon'].str.to_uppercase())
    raw_rpf = raw_rpf.filter(~raw_rpf['codon'].str.contains('N'))

    return raw_rpf


def get_frame_rpf(raw_rpf, sample_name, frame):

    '''
    @Message  : merge all frame density or retrieve one of them
    @Input    : raw_rpf --> rpf table
                merged_rpf --> merged three frame rpf table 
                sample_name --> sample name of rpf table 
                frame --> frame of  [0, 1, 2, all] to retrieve
    @Return   : merged_rpf --> retrieve specific frame of rpf table
    '''
    
    merged_rpf = raw_rpf[:, 0:6]

    if frame == 'all':
        for now_sp in sample_name:
            print('Import the density file {file_name}.'.format(file_name=now_sp), flush=True)
            now_sp_index = [now_sp + '_f0', now_sp + '_f1', now_sp + '_f2']
            
            # merged_rpf = merged_rpf.with_columns(raw_rpf[now_sp_index].sum(axis=1)).rename({now_sp_index[0]: now_sp})
            # report a bug in polars, the sum function is not working.
            merged_rpf = merged_rpf.with_columns(raw_rpf[:, now_sp_index[0]] + raw_rpf[:, now_sp_index[1]] + raw_rpf[:, now_sp_index[2]]).rename({now_sp_index[0]: now_sp})
            
    elif frame == '0':
        for now_sp in sample_name:
            print('Import the density file {file_name}.'.format(file_name=now_sp), flush=True)
            now_sp_index = now_sp + '_f0'
            merged_rpf = merged_rpf.with_columns(raw_rpf[:, now_sp_index]).rename({now_sp_index: now_sp})

    elif frame == '1':
        for now_sp in sample_name:
            print('Import the density file {file_name}.'.format(file_name=now_sp), flush=True)
            now_sp_index = now_sp + '_f1'
            merged_rpf = merged_rpf.with_columns(raw_rpf[:, now_sp_index]).rename({now_sp_index: now_sp})

    elif frame == '2':
        for now_sp in sample_name:
            print('Import the density file {file_name}.'.format(file_name=now_sp), flush=True)
            now_sp_index = now_sp + '_f2'
            merged_rpf = merged_rpf.with_columns(raw_rpf[:, now_sp_index]).rename({now_sp_index: now_sp})

    merged_rpf = merged_rpf.to_pandas()

    return merged_rpf


def set_codon_shift(sites):

    '''
    @Message  : set the codon shift by E/P/A site
    @Input    : sites --> [E, P, A] sites in the ribosome
    @Return   : shift_codon --> [-1, 0, 1] number for [E, P, A] site to shift.
    '''
    
    if sites == 'E':
        shift_codon = -1
    elif sites == 'P':
        shift_codon = 0
    elif sites == 'A':
        shift_codon = 1
    else:
        shift_codon = 0
    return shift_codon


def shift_site(merged_rpf, sample_name, shift_num):
    '''
    @Message  : to shift the E/P/A-site.
    @Input    : merged_rpf --> shift codon from default P-site to E/A site
    @Return   : merged_rpf --> shift to specific coodn site of rpf table
    '''
    
    merged_rpf.loc[:, sample_name] = merged_rpf.groupby('name')[sample_name].shift(shift_num).fillna(0).values
    # merged_rpf = merged_rpf.with_columns(merged_rpf.groupby('name').apply(lambda x: x[sample_name].shift(-1)))
    
    return merged_rpf


def sum_total_rpf(raw_rpf):
    # total_rpf_num = rpf[sample_name].sum()

    total_rpf_num = raw_rpf.select(raw_rpf.columns[6:]).sum().to_pandas().T.reset_index()
    total_rpf_num.columns = ['sample', 'total_rpf']
    total_rpf_num['sample'] = total_rpf_num['sample'].str[:-3]
    total_rpf_num = total_rpf_num.groupby('sample')['total_rpf'].sum()

    return total_rpf_num


def high_expression(merged_rpf, total_rpf_num, sample_name, rpf_num):

    '''
    @Message  : filter the high expression gene.
    @Input    : merged_rpf --> merged rpf table
                sample_name --> sample name of rpf table
                rpf_num --> minimum value for rpf table filter
    @Return   : total_rpf_num --> total rpf of each sample
                gene_rpf_sum --> total rpf of each gene
                high_gene --> retrieved gene list with more than minimum rpf value
                high_rpf --> retrieved gene rpf table with more than minimum rpf value
    '''

    gene_rpf_sum = merged_rpf.groupby('name')[sample_name].sum()
    high_gene = gene_rpf_sum.loc[gene_rpf_sum.mean(axis=1) >= rpf_num, :].index
    high_rpf = merged_rpf.loc[merged_rpf['name'].isin(high_gene), :]

    return gene_rpf_sum, high_gene, high_rpf


def sum_sample_number(raw_rpf, sample_num):
    '''
    @Message  : statistic the number of samples.
    @Input    : raw_rpf --> rpf table in polars dataframe format
                sample_num --> default [None]
    @Return   : sample_num --> number of samples
    '''
    
    if sample_num:
        pass
    else:
        sample_num = int((len(raw_rpf.columns) - 6) / 3)

    return sample_num


def retrieve_sample_name(raw_rpf, sample_name):
    '''
    @Message  : retrieve the rpf table with specific sample name.
    @Input    : sample_name --> input sample name, default [None]
    @Return   : sample_name --> retrieved sample name from rpf table
    '''
    
    if sample_name:
        pass
    else:
        # sample_name = pl.Series(raw_rpf.columns[6::]).apply(lambda s: s[:-3]).unique().to_list()
        sample_name = pd.Series(raw_rpf.columns[6::]).str[:-3].drop_duplicates().to_list()

    return sample_name


def filter_tis_tts(raw_rpf, tis, tts):
    '''
    @Message  : trim the rpf table with tis offset.
    @Input    : raw_rpf --> raw rpf table
                tis --> offset from translation initiation site
                tts --> offset from translation termination site
    @Return   : raw_rpf --> trimmed rpf table
    '''
    
    if not tis is None:
        raw_rpf = raw_rpf.filter(pl.col("from_tis") >= tis)
    else:
        pass

    if not tts is None:
        raw_rpf = raw_rpf.filter(pl.col("from_tts") <= -tts)
    else:
        pass

    return raw_rpf


def retrieve_gene_rpf(raw_rpf, gene):
    '''
    @Message  : retrieve the rpf table with specific gene ids.
    @Input    : raw_rpf --> input rpf in polars dataframe format
                gene --> input gene ids, default [None]
    @Return   : sample_name --> retrieved gene rpf from merged rpf table
    '''

    if gene:
        gene_csv = pd.read_csv(gene, header=0, index_col=None, sep='\t')
        if 'transcript_id' in gene_csv.columns:
            gene_name = gene_csv['transcript_id'].tolist()
        else:
            gene_name = gene_csv.iloc[:, 0].tolist()

        raw_rpf = raw_rpf.filter(pl.col("name").is_in(gene_name))
    else:
        pass
    
    return raw_rpf


def check_merged_rpf_results(merged_rpf):
    if merged_rpf.empty:
        print('Filtered density table is empty! Please recheck the parameters and data quality.', flush=True)
        sys.exit()


def check_high_rpf_results(high_rpf):
    if high_rpf.empty:
        print('Filtered density table is empty! Please recheck the parameters and data quality.', flush=True)
        sys.exit()


def rpf_to_rpm(raw_rpf, norm):
    '''
    @Message  : convert rpf to rpm value.
    @Input    : raw_rpf --> rpf table in polars dataframe
                norm --> default [False]
    @Return   : raw_rpf --> rpm table
    '''
    
    if norm:
        raw_rpf.iloc[:, 6:] = raw_rpf.iloc[:, 6:] * 1000000 / raw_rpf[:, 6:].sum()
    else:
        pass
    
    return raw_rpf


def import_rpf(rpf_file, sample_num, sample_name, sites, frame, gene, rpf_num, tis = None, tts = None):
    ########################################
    # import the rpf file generate by ribo_parser
    raw_rpf = read_rpf_file(rpf_file)
    total_rpf_num = sum_total_rpf(raw_rpf)

    ########################################
    # User can specify a gene list in txt format directly to exclude genes that don't want to analyze.
    # and the gene list like this (no column names are required):

    # YDL246C
    # YDL243C
    # YDR387C
    # ...
    raw_rpf = retrieve_gene_rpf(raw_rpf, gene)

    ########################################
    # filter the CDS range of density file (tis, tts)
    raw_rpf = filter_tis_tts(raw_rpf, tis, tts)
    valid_rpf_num = sum_total_rpf(raw_rpf)

    ########################################
    # retrieve the sample name and numbers
    # columns like this ['sample1_f0', 'sample1_f1', 'sample1_f2']
    sample_num = sum_sample_number(raw_rpf, sample_num)
    sample_name = retrieve_sample_name(raw_rpf, sample_name)

    ########################################
    # merge the RPFs of nucleotide to codon
    # retrieve the codon frame
    # front 6 columns ['name', 'now_nt', 'from_tis', 'from_tes', 'region', 'codon']
    merged_rpf = get_frame_rpf(raw_rpf, sample_name, frame)
    # total_rpf_num = sum_total_rpf(merged_rpf, sample_name)
    

    ########################################
    # RPFs are different at E/P/A site. Default P-site. shift -1/1 codon to E-site/A-site
    shift_num = set_codon_shift(sites)
    merged_rpf = shift_site(merged_rpf, sample_name, shift_num)

    ########################################
    # Filter the high expression genes by specified rpfs num with at least one sample.
    gene_rpf_sum, high_gene, high_rpf = high_expression(merged_rpf, total_rpf_num, sample_name, rpf_num)

    ########################################
    # Check the filtered results. exit if the dataframe is empty.
    check_merged_rpf_results(merged_rpf)
    check_high_rpf_results(high_rpf)

    return raw_rpf, sample_name, sample_num, merged_rpf, total_rpf_num, gene_rpf_sum, high_gene, high_rpf


def codon_table():
    codon_anno = {
        'AAA': ['Lys', 'K'], 'AAC': ['Asn', 'N'], 'AAG': ['Lys', 'K'], 'AAT': ['Asn', 'N'],
        'ACA': ['Thr', 'T'], 'ACC': ['Thr', 'T'], 'ACG': ['Thr', 'T'], 'ACT': ['Thr', 'T'],
        'AGA': ['Arg', 'R'], 'AGC': ['Ser', 'S'], 'AGG': ['Arg', 'R'], 'AGT': ['Ser', 'S'],
        'ATA': ['Ile', 'I'], 'ATC': ['Ile', 'I'], 'ATG': ['Met', 'M'], 'ATT': ['Ile', 'I'],
        'CAA': ['Gln', 'Q'], 'CAC': ['HIS', 'H'], 'CAG': ['Gln', 'Q'], 'CAT': ['HIS', 'H'],
        'CCA': ['Pro', 'P'], 'CCC': ['Pro', 'P'], 'CCG': ['Pro', 'P'], 'CCT': ['Pro', 'P'],
        'CGA': ['Arg', 'R'], 'CGC': ['Arg', 'R'], 'CGG': ['Arg', 'R'], 'CGT': ['Arg', 'R'],
        'CTA': ['Leu', 'L'], 'CTC': ['Leu', 'L'], 'CTG': ['Leu', 'L'], 'CTT': ['Leu', 'L'],
        'GAA': ['Glu', 'E'], 'GAC': ['Asp', 'D'], 'GAG': ['Glu', 'E'], 'GAT': ['Asp', 'D'],
        'GCA': ['Ala', 'A'], 'GCC': ['Ala', 'A'], 'GCG': ['Ala', 'A'], 'GCT': ['Ala', 'A'],
        'GGA': ['Gly', 'G'], 'GGC': ['Gly', 'G'], 'GGG': ['Gly', 'G'], 'GGT': ['Gly', 'G'],
        'GTA': ['Val', 'V'], 'GTC': ['Val', 'V'], 'GTG': ['Val', 'V'], 'GTT': ['Val', 'V'],
        'TAC': ['Tyr', 'Y'], 'TAT': ['Tyr', 'Y'], 'TCA': ['Ser', 'S'], 'TCC': ['Ser', 'S'],
        'TCG': ['Ser', 'S'], 'TCT': ['Ser', 'S'], 'TGC': ['Cys', 'C'], 'TGG': ['Trp', 'W'],
        'TGT': ['Cys', 'C'], 'TTA': ['Leu', 'L'], 'TTC': ['Phe', 'F'], 'TTG': ['Leu', 'L'],
        'TTT': ['Phe', 'F'], 'TAA': ['Stop', '*'], 'TAG': ['Stop', '*'], 'TGA': ['Stop', '*']
    }

    codon_df = pd.DataFrame(codon_anno, index=['AA', 'Abbr']).T.sort_values('Abbr')

    codon_df.index.name = 'codon'
    # codon_df = codon_df.reset_index()

    return codon_anno, codon_df



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : rna_density.py


from .ribo import ArgsParser
from .ribo import RNA


def main():
    ArgsParser.now_time()
    print('\nConvert reads to reads density.\n', flush=True)
    print('Step1: Checking the input Arguments.', flush=True)
    args = ArgsParser.rna_args_parser()

    rna_attr = RNA.RNA(args)

    print('\nStep2: Import the P-site offset.', flush=True)
    rna_attr.read_offset()

    print('\nStep3: Import the transcripts annotation.', flush=True)
    rna_attr.read_transcript()

    print('\nStep4: Import the BAM file.', flush=True)
    rna_attr.read_bam()
    rna_attr.calculate_density()

    print('\nStep5: Format the in-frame reads density.', flush=True)
    rna_attr.run_format_reads_with_multi_thread()

    print('\nStep6: Output the reads density.', flush=True)
    rna_attr.output_density()

    print('\nAll done.', flush=True)
    ArgsParser.now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : BamFilter.py


import pysam
import os
import sys


class BamFilter():

    def __init__(self, args):
        
        # filename
        self.ibam = args.ibam
        self.obam = args.obam

        # option for reads filter
        self.length = args.length
        self.quality = args.quality
        self.secondary = args.unique
        self.supplementary = args.supplementary

        # file store
        self.input_bam = None
        self.flt_bam = None


    def get_length_list(self):

        self.length = [int(i) for i in self.length.split(',')]


    def filter_reads(self):
        """
        filte the mapped reads with three condition
        
        secondary mapping
        supplymentary mapping
        high quality
        fitted length

        """
        for read in self.input_bam.fetch(until_eof=True):

            if read.is_unmapped:
                continue

            if read.is_secondary != self.secondary:
                continue

            if read.is_supplementary != self.supplementary:
                continue

            if read.mapping_quality < self.quality:
                continue

            if read.infer_read_length() in self.length:
                self.flt_bam.write(read)

    def import_bam(self):
        """
        check the file format
        import the alignment file with bam or sam file

        """
        sample_suffix = os.path.splitext(self.ibam)[1]

        if sample_suffix.lower() == ".bam":
            print("import file: {bam}.\n".format(bam=self.ibam), flush=True)
            read_format = 'rb'
        elif sample_suffix.lower() == ".sam":
            print("import file: {bam}.\n".format(bam=self.ibam), flush=True)
            read_format = 'r'
        else:
            print("Unknown file format, please input the correct bam or sam file.", flush=True)
            sys.exit()

        self.input_bam = pysam.AlignmentFile(self.ibam, read_format)

        self.flt_bam = pysam.AlignmentFile(self.obam, "wb", template=self.input_bam)

        self.get_length_list()
        self.filter_reads()

        self.input_bam.close()
        self.flt_bam.close()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : MetaCodon.py


# from keyword import kwlist
# from concurrent.futures import ThreadPoolExecutor

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

from collections import OrderedDict
import seaborn as sns

from scipy.signal import savgol_filter

from . import RPFs


class MetaCodon(object):

    def __init__(self, args):
        # input and output
        self.ribo = args.rpf
        self.gene = args.list
        self.output = args.output

        # parameters for rpfs filter
        self.frame = args.frame
        self.rpf_num = args.min
        self.tis = args.tis
        self.tts = args.tts
        self.norm = args.normal

        # specified codon for meta-codon plot
        # self.thread = args.thread

        self.codon = args.codon
        self.codon_list = None
        self.around = args.around

        self.density_df = OrderedDict()
        self.sequence_df = OrderedDict()

        # format the meta-codon
        self.scale = args.scale
        self.unique = args.unique

        # smooth the meta-codon
        self.smooth = args.smooth

        # output the figures
        self.fig = args.fig

        # groups
        self.sample_name = None
        self.sample_num = None
        self.total_rpf_num = None
        self.high_rpf = None

    def import_codon(self):
        if self.codon:
            self.codon_list = pd.read_csv(self.codon,
                                          index_col=False,
                                          header=None,
                                          sep='\t')
            self.codon_list.columns = ['codon']
            self.codon_list['codon'] = self.codon_list['codon'].str.upper()
            self.codon_list['codon'] = self.codon_list['codon'].str.replace('U', 'T')
        else:
            codon_anno, codon_df = RPFs.codon_table()
            self.codon_list = codon_df.reset_index()

    def import_rpf(self):
        '''
        1. import the rpf density
        2. filter the high expressiong genes
        3. convert the rpf density to rpm density
        4. scale the rpf density with each gene expression

        '''
        # raw_rpf, sample_name, sample_num, merged_rpf, total_rpf_num, gene_rpf_sum, high_gene, high_rpf
        rpf_results = RPFs.import_rpf(rpf_file=self.ribo,
                                      sites='A',
                                      frame=self.frame,
                                      sample_num=None,
                                      sample_name=None,
                                      tis = self.tis,
                                      tts = self.tts,
                                      gene=self.gene,
                                      rpf_num=self.rpf_num)
        # self.raw_rpf = rpf_results[0]
        self.sample_name = rpf_results[1]
        self.sample_num = rpf_results[2]
        self.merged_rpf = rpf_results[3]
        self.total_rpf_num = rpf_results[4]
        self.gene_rpf_sum = rpf_results[5]
        self.high_gene = rpf_results[6]
        self.high_rpf = rpf_results[7]

        del rpf_results

        if self.norm:
            self.merged_rpf[self.sample_name] = self.merged_rpf[self.sample_name] * 1e6 / self.total_rpf_num
            self.high_rpf = self.merged_rpf.loc[self.merged_rpf['name'].isin(self.high_gene)]
            self.gene_rpf_sum = self.high_rpf.loc[:, ["name"] + self.sample_name].groupby("name")[self.sample_name].sum()

        if self.scale:
            # merge the gene expression with rpf density
            self.gene_rpf_sum = self.gene_rpf_sum.add_suffix("_sum")
            self.high_rpf = pd.merge(self.high_rpf, self.gene_rpf_sum, on = "name")

            # create the rpm dataframe and rpm_sum dataframe
            np.seterr(divide='ignore', invalid='ignore')
            rpm = self.high_rpf[self.sample_name]
            rpm_sum = self.high_rpf[self.gene_rpf_sum.columns]
            rpm_sum.columns = self.sample_name
            
            # scale the rpf density with each gene expression
            self.high_rpf[self.sample_name] = np.divide(rpm, rpm_sum)
            self.high_rpf = self.high_rpf.drop(columns = self.gene_rpf_sum.columns)

    def smooth_rpf_density(self):
        '''
        Smooth the rpf density with the mean of rpf density.

        RPFs density is a discrete distribution, which is not suitable for meta-codon plot.

        1. set the windows and power of smooth
        2. smooth the rpf with the mean of rpf density of each gene and sample
        '''
        
        self.smoothed_rpf = pd.DataFrame()

        if self.smooth:
            self.window = int(self.smooth.split(',')[0])
            self.power = int(self.smooth.split(',')[1])

            rpf_grouped = self.high_rpf.groupby('name')

            for gene, group in rpf_grouped:
                for colname in self.sample_name:
                    group[colname] = savgol_filter(group[colname], self.window, self.power)
                
                self.smoothed_rpf = pd.concat([self.smoothed_rpf, group])
        
            del self.high_rpf
            self.high_rpf = self.smoothed_rpf

        else:
            pass

    def delete_repeat_codon(self, codon_idx_true):
        '''
        Check the repeat codon in the range of specific codon window,
        Because, if the same codon in the same window, it will introduce the voice in the background.
        A clean background makes the results more reliable.

        1. delete the repeat codon in same window:
        [..., AAA, TGT, CAC, AAA, GGG, ATG, TGC, CTG, ...] ==> delete the 'AAA' codon site

        '''

        codon_idx_delta = (codon_idx_true.index[1:] - codon_idx_true.index[:-1]) > self.around * 2
        codon_idx_delta1_list = [True] + codon_idx_delta.tolist()
        codon_idx_delta2_list = codon_idx_delta.tolist() + [True]

        uniq_codon_list = [a and b for a, b in zip(codon_idx_delta1_list, codon_idx_delta2_list)]
        if len(uniq_codon_list) < 0:
            return 0, 0
        else:
            uniq_codon_idx = codon_idx_true.loc[uniq_codon_list]
            uniq_site_num = uniq_codon_idx.sum()

            return uniq_codon_idx, uniq_site_num

    def delete_out_of_range_site(self, uniq_codon_idx):
        '''
        reterieve the codon windows may out of the range of CDS region
        so we need to check the limits of codon windows.
        like the codon [GCT]:

        ACACAGTCGTACGTAGCA  GCT  AGCGAG[TGA]

        The GCT codon is located close to the stop codon, making it impossible to take a complete width sliding window centered on it.
        '''

        left_limit = uniq_codon_idx.index - self.around - 4
        right_limit = uniq_codon_idx.index + self.around + 4

        left_limit = left_limit[left_limit.isin(self.high_rpf.index)] + self.around + 4
        right_limit = right_limit[right_limit.isin(self.high_rpf.index)] - self.around - 4

        uniq_codon_idx = left_limit.intersection(right_limit)

        return uniq_codon_idx

    def scale_density(self, density_merge):
        '''
        scale the density with the mean of density
        '''
        if self.scale:
            density_merge = np.divide(density_merge, density_merge.mean())
            return density_merge
        else:
            return density_merge

    def single_codon(self, codon):
        '''
        match single codon and retrieve the density
        1. reterieve window: [-20, 0, 20]
        
        2. delete the repeat codon in same window:
        [..., AAA, TGT, CAC, AAA, GGG, ATG, TGC, CTG, ...] ==> delete the 'AAA' codon site
        
        3. reterieve the codon density by the window
        
        4. merge all results

        '''

        # retrieve the codon index and exclude the codon site out of range
        codon_idx = (self.high_rpf['codon'] == codon) & (self.high_rpf['from_tis'] > self.around) & (self.high_rpf['from_tts'] < -self.around)
        codon_idx_true = codon_idx[codon_idx == True]
        site_num = sum(codon_idx)
        
        # exit if none fitted results
        if site_num == 0:
            return

        # delete the Cross repetition codon in same window
        if self.unique:
            uniq_codon_idx, uniq_site_num = self.delete_repeat_codon(codon_idx_true)
        else:
            uniq_codon_idx = codon_idx_true
            uniq_site_num = site_num
        
        # exit if none fitted results
        if uniq_site_num == 0:
            return

        # delete the codon site out of range
        # uniq_codon_idx = self.delete_out_of_range_site(uniq_codon_idx)

        # exit if none fitted results
        if len(uniq_codon_idx) == 0:
            return

        # retrieve the codon density codon by codon
        density_df = []
        sequence_df = []
        
        # retrieve the codon in upstream/downstream window [-20, 0, 20]
        # for posi in range(-self.around, self.around + 1): # this is wrong, should be -21, -1, 19
        for posi in range(-self.around, self.around + 1):
            idx_tmp = uniq_codon_idx.index + posi
            tmp_dst = self.high_rpf.loc[idx_tmp, self.sample_name].mean()
            density_df.append(tmp_dst)

            tmp_seq = self.high_rpf.loc[idx_tmp, ['name', 'codon']]
            tmp_seq.index -= posi
            tmp_seq.columns = ['name', posi]
            sequence_df.append(tmp_seq)

        # merge meta RPFs
        density_merge = pd.concat(density_df, axis = 1).T
        density_merge = self.scale_density(density_merge)
        density_merge.index = np.arange(-self.around, self.around + 1)
        density_merge.insert(loc=0, column='Frame', value=self.frame)
        if self.frame == 'all':
            nt_index = density_merge.index * 3
            density_merge.insert(loc=0, column='Nucleotide', value=nt_index)
        else:
            nt_index = density_merge.index * 3 + int(self.frame)
            density_merge.insert(loc=0, column='Nucleotide', value=nt_index)

        self.density_df[codon] = [site_num, uniq_site_num, density_merge]

        # merge meta sequences
        sequence_merge = pd.concat(sequence_df, axis=1)
        sequence_merge = sequence_merge.loc[:, ~sequence_merge.columns.duplicated()]
        sequence_merge.reindex(columns=np.array(range(-self.around, self.around + 1)).tolist())
        self.sequence_df[codon] = [site_num, uniq_site_num, sequence_merge]

    def multiple_codon(self, codon, codon_num):
        '''
        match multiple codon and retrieve the density
        1. 
        reterieve first codon index: [4, 26, 79, 148, 205]

        reterieve second codon index - 1 : [5-1, 27-1, 89-1, 135-1, 206-1]
        
        2. multiple codons should then be aligned to the same index:
        
        aligend index is : [4, 26, 205]

        3. reterieve the codon window

        4. delete the repeat codon in same window:
        [..., AAAAAA, TGTTGT, CACCAC, AAAAAA, GGGGGG, ATGATG, TGCTGC, CTGCTG, ...] ==> delete the 'AAAAAA' codon site
        
        5. reterieve the codon density by the window
        
        6. merge all results

        '''

        print('Now is: {codon}.'.format(codon=codon), flush=True)

        # match first and second codon index
        idx_list = []
        shift_codon = 0
        for f0 in range(0, len(codon), 3):
            single_codon = codon[f0:f0 + 3]

            idx = (self.high_rpf['codon'] == single_codon) & (self.high_rpf['from_tis'] > self.around) & (self.high_rpf['from_tts'] < -self.around)
            idx = idx.shift(shift_codon, fill_value=False)
            shift_codon -= 1

            idx_list.append(idx)

        # merge multiple codon index
        idx_mer = pd.concat(idx_list, axis=1)
        idx = idx_mer.sum(axis=1)
        idx = idx.apply(lambda x: True if x == codon_num else False)
        site_num = sum(idx)

        # exit if none fitted results
        if site_num == 0:
            return

        # delete the Cross repetition codon in same window
        if self.unique:
            uniq_codon_idx, uniq_site_num = self.delete_repeat_codon(idx[idx])
        else:
            uniq_codon_idx = idx
            uniq_site_num = site_num
        
        # exit if none fitted results
        if uniq_site_num == 0:
            return

        # delete the codon site out of range
        # uniq_codon_idx = self.delete_out_of_range_site(uniq_codon_idx)
        
        # exit if none fitted results
        if len(uniq_codon_idx) == 0:
            return

        # retrieve the codon density codon by codon
        density_df = []
        sequence_df = []

        for posi in range(-self.around, self.around + codon_num):
            idx_tmp = uniq_codon_idx.index + posi
            
            tmp_dst = self.high_rpf.loc[idx_tmp, self.sample_name].mean()
            density_df.append(tmp_dst)

            tmp_seq = self.high_rpf.loc[idx_tmp, ['name', 'codon']]
            tmp_seq.index -= posi
            tmp_seq.columns = ['name', posi]
            sequence_df.append(tmp_seq)

        # merge meta RPFs
        density_merge = pd.concat(density_df, axis=1).T
        density_merge = self.scale_density(density_merge)

        density_merge.index = np.arange(-self.around, self.around + codon_num)
        density_merge.insert(loc=0, column='Frame', value=self.frame)
        if self.frame == 'all':
            nt_index = density_merge.index * 3
            density_merge.insert(loc=0, column='Nucleotide', value=nt_index)
        else:
            nt_index = density_merge.index * 3 + int(self.frame)
            density_merge.insert(loc=0, column='Nucleotide', value=nt_index)

        self.density_df[codon] = [site_num, uniq_site_num, density_merge]

        sequence_merge = pd.concat(sequence_df, axis=1)
        sequence_merge = sequence_merge.loc[:, ~sequence_merge.columns.duplicated()]
        self.sequence_df[codon] = [site_num, uniq_site_num, sequence_merge]
        
    def reterieve_codon_density(self):
        from concurrent.futures import ThreadPoolExecutor

        for codon in self.codon_list['codon']:
            print('Now is: {codon}.'.format(codon=codon), flush=True)
            # skip codon does not fit 3-nt periodicity
            if len(codon) % 3 != 0:
                print('Skip! {codon} does not fit 3-nt periodicity!'.format(codon=codon), flush=True)
                continue

            codon_num = int(len(codon) / 3)
            if codon_num == 1:
                self.single_codon(codon)
            elif codon_num > 1:
                self.multiple_codon(codon, codon_num)
            else:
                print('Skip! {codon} does not fit 3-nt periodicity!'.format(codon=codon), flush=True)

    def output_meta_codon_density(self):
        for codon, mess in self.density_df.items():
            mess[2].index.name = 'Codon'
            mess[2].to_csv(self.output + '_' + codon + '_' + str(mess[0]) + '_' + str(mess[1]) + '_meta_density.txt', sep='\t', index=True)
            mess[2] = mess[2].drop(columns = ['Nucleotide', 'Frame'])
    
    def output_meta_codon_seq(self):
        for codon, mess in self.sequence_df.items():
            mess[2].index.name = 'Codon'
            mess[2].to_csv(self.output + '_' + codon + '_' + str(mess[0]) + '_' + str(mess[1]) + '_meta_sequence.txt', sep='\t', index=True)
    
    def draw_meta_codon(self):
        # draw the line meta plot of each codon
        if self.fig:

            for codon, mess in self.density_df.items():
                out_pdf = self.output + '_' + codon + '.pdf'
                out_png = self.output + '_' + codon + '.png'

                # if self.scale:
                #     mess[2] = mess[2]/mess[2].mean()

                matplotlib.use('AGG')
                fig, ax = plt.subplots(figsize=(8, 5), dpi=300)
                ax = sns.lineplot(data=mess[2], linewidth=1.2, ax=ax)

                # ax.set_xlim(left=-20, right=20)
                ax.autoscale(enable=True, axis='y')

                plt.legend(loc='center left',
                        bbox_to_anchor=(1, 0.5),
                        fancybox=True,
                        shadow=True,
                        ncol=1)

                ax.set_title(codon + '(' + str(mess[0]) + ', ' + str(mess[1]) + ')')
                ax.set_xlabel('Distance from codon')

                # set the ylabel
                if self.norm:
                    if self.scale:
                        ax.set_ylabel('Scaled mean RPM density')
                    else:
                        ax.set_ylabel('Mean RPM density')
                else:
                    if self.scale:
                        ax.set_ylabel('Scaled mean RPFs density')
                    else:
                        ax.set_ylabel('Mean RPFs density')

                plt.tight_layout()
                # plt.show()

                fig.savefig(fname=out_pdf)
                fig.savefig(fname=out_png)
                plt.close()
        else:
            pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# @Script  : rpf_Shuffle.py


from .ribo import ArgsParser
from .ribo.Shuffle import *


def main():
    ArgsParser.now_time()
    print('Shuffle the RPFs data.\n', flush=True)
    print('Step1: Checking the input Arguments.\n', flush=True)
    args = ArgsParser.shuffle_args_parser()
    rpfs = Shuffle(args)

    print('Step2: Import the RPFs.\n', flush=True)
    rpfs.import_rpf()

    print('Step3: Shuffle the RPFs table.\n', flush=True)
    rpfs.shuffle_rpfs()

    print('Step4: Output the RPFs table.\n', flush=True)
    rpfs.output_rpfs()

    ArgsParser.now_time()
    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Project      : riboParser
@Script       : line_feed.py
@Environment  : python 3.8.5
@Version      : 1.0
@Author       : Rensc 
@Time         : 2023/11/07 18:00:49
@E-mail       : rensc0718@163.com
@License      : (C)Copyright 2023-2025, Ren Shuchao
'''


# from Bio import SeqIO
import argparse


def line_feed_args_parser():
    
    parser = argparse.ArgumentParser(description='This script is used to convert multiple lines to one line.')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        '-i', dest='input', required=True, type=str, help='input fasta file name.'
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='output fasta file name.'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def line_feed(args):
    '''
    @Message  : function for convert multiple lines to one line.
    @Input    : fasta --> input file
    @Return   : output --> fasta file
    '''

    with open(args.input, 'r') as fa_in:
        with open(args.output, 'w') as fa_out:
            seq_id = ''
            seq_mess = ''

            for line in fa_in:
                if line.startswith('>') and seq_mess == '':
                    seq_id = line.strip()

                elif line.startswith('>') and seq_mess != '':
                    fa_out.writelines(seq_id + '\n')
                    fa_out.writelines(seq_mess + '\n')

                    seq_id = line.strip()
                    seq_mess = ''
                else:
                    seq_mess += line.strip()
                
            fa_out.writelines(seq_id + '\n')
            fa_out.writelines(seq_mess + '\n')


def main():
    args = line_feed_args_parser()
    line_feed(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : rpf_corr.py


import matplotlib
import matplotlib.pyplot as plt
import pandas as pd
import polars as pl
import seaborn as sns

from .ribo import ArgsParser


def read_rpf(rpf_file):
    rpf = pl.read_csv(rpf_file, separator='\t', has_header=True)
    rpf = rpf.to_pandas()
    # cds = rpf[rpf["region"] == "cds"]
    sample_num = int((len(rpf.columns) - 6) / 3)
    sample_name = pd.Series(rpf.columns[6::]).str[:-3].drop_duplicates().tolist()
    frame_rpf = pd.DataFrame()
    frame0_rpf, frame1_rpf, frame2_rpf = pd.DataFrame(), pd.DataFrame(), pd.DataFrame()

    # split the frame
    for now_sp in sample_name:
        print('Import the RPFs file {file_name}.'.format(file_name=now_sp), flush=True)
        now_sp_index = [now_sp + '_f0', now_sp + '_f1', now_sp + '_f2']
        frame_rpf[now_sp] = rpf.loc[:, now_sp_index].sum(axis=1)
        frame0_rpf[now_sp] = rpf.loc[:, now_sp_index].iloc[:, 0]
        frame1_rpf[now_sp] = rpf.loc[:, now_sp_index].iloc[:, 1]
        frame2_rpf[now_sp] = rpf.loc[:, now_sp_index].iloc[:, 2]

    # calculate the rpm
    total_rpf = frame_rpf.sum()
    frame_rpm = frame_rpf.div(total_rpf) * 1e6
    frame0_rpm = frame0_rpf.div(total_rpf) * 1e6
    frame1_rpm = frame1_rpf.div(total_rpf) * 1e6
    frame2_rpm = frame2_rpf.div(total_rpf) * 1e6

    return rpf, sample_name, frame_rpm, frame0_rpm, frame1_rpm, frame2_rpm


def calc_gene_rpm(rpf, sample_name, frame_rpm, frame0_rpm, frame1_rpm, frame2_rpm):
    # add the annotation
    merge_f_rpm = pd.concat([rpf.iloc[:, 0:5], frame_rpm], axis=1)
    merge_f0_rpm = pd.concat([rpf.iloc[:, 0:5], frame0_rpm], axis=1)
    merge_f1_rpm = pd.concat([rpf.iloc[:, 0:5], frame1_rpm], axis=1)
    merge_f2_rpm = pd.concat([rpf.iloc[:, 0:5], frame2_rpm], axis=1)

    # summary the gene rpm
    cds_f_rpm = merge_f_rpm[merge_f_rpm["region"] == "cds"
                            ][["name"] + sample_name].groupby("name")[sample_name].apply(sum)
    cds_f0_rpm = merge_f0_rpm[merge_f0_rpm["region"] == "cds"
                              ][["name"] + sample_name].groupby("name")[sample_name].apply(sum)
    cds_f1_rpm = merge_f1_rpm[merge_f1_rpm["region"] == "cds"
                              ][["name"] + sample_name].groupby("name")[sample_name].apply(sum)
    cds_f2_rpm = merge_f2_rpm[merge_f2_rpm["region"] == "cds"
                              ][["name"] + sample_name].groupby("name")[sample_name].apply(sum)

    # calculate the correlation of samples at gene level
    gene_corr_f = cds_f_rpm.corr(method='pearson')
    gene_corr_f0 = cds_f0_rpm.corr(method='pearson')
    gene_corr_f1 = cds_f1_rpm.corr(method='pearson')
    gene_corr_f2 = cds_f2_rpm.corr(method='pearson')

    gene_corr_f.to_csv('gene_corr_frame.txt', sep='\t', index=True)
    gene_corr_f0.to_csv('gene_corr_f0.txt', sep='\t', index=True)
    gene_corr_f1.to_csv('gene_corr_f1.txt', sep='\t', index=True)
    gene_corr_f2.to_csv('gene_corr_f2.txt', sep='\t', index=True)

    return gene_corr_f, gene_corr_f0, gene_corr_f1, gene_corr_f2


def calc_rpf_rpm(frame_rpm, frame0_rpm, frame1_rpm, frame2_rpm):
    # calculate the correlation of samples at rpf level
    rpf_corr_f = frame_rpm.corr(method='pearson')
    rpf_corr_f0 = frame0_rpm.corr(method='pearson')
    rpf_corr_f1 = frame1_rpm.corr(method='pearson')
    rpf_corr_f2 = frame2_rpm.corr(method='pearson')

    rpf_corr_f.to_csv('rpf_corr_frame.txt', sep='\t', index=True)
    rpf_corr_f0.to_csv('rpf_corr_f0.txt', sep='\t', index=True)
    rpf_corr_f1.to_csv('rpf_corr_f1.txt', sep='\t', index=True)
    rpf_corr_f2.to_csv('rpf_corr_f2.txt', sep='\t', index=True)

    return rpf_corr_f, rpf_corr_f0, rpf_corr_f1, rpf_corr_f2


def draw_corr_plot(frame, frame0, frame1, frame2, types, out_prefix):
    # output figure name
    out_pdf = out_prefix + "_" + types + "_correlation_plot.pdf"
    out_png = out_prefix + "_" + types + "_correlation_plot.png"
    num = len(frame)

    # draw the figure
    matplotlib.use('AGG')

    fig, axes = plt.subplots(2, 2, figsize=(10 + num * 0.3, 9 + num * 0.25), dpi=300)
    ax1 = sns.heatmap(data=frame, cmap="Blues",  # vmin=-1, vmax=1, center=0,
                      xticklabels=1, yticklabels=1, ax=axes[0, 0])
    ax1.set_title('all frame rpf', fontsize=20)
    ax2 = sns.heatmap(data=frame0, cmap="Blues",  # vmin=-1, vmax=1, center=0,
                      robust=True, xticklabels=1, yticklabels=1, ax=axes[0, 1])
    ax2.set_title('frame0 rpf', fontsize=20)
    ax3 = sns.heatmap(data=frame1, cmap="Blues",  # vmin=-1, vmax=1, center=0,
                      robust=True, xticklabels=1, yticklabels=1, ax=axes[1, 0])
    ax3.set_title('frame1 rpf', fontsize=20)
    ax4 = sns.heatmap(data=frame2, cmap="Blues",  # vmin=-1, vmax=1, center=0,
                      robust=True, xticklabels=1, yticklabels=1, ax=axes[1, 1])
    ax4.set_title('frame2 rpf', fontsize=20)

    plt.tight_layout()
    # plt.show()
    fig.savefig(fname=out_pdf)
    fig.savefig(fname=out_png)


def main():
    ArgsParser.now_time()
    print('\nDraw the correlation of samples.\n', flush=True)
    print('\nStep1: Checking the input Arguments.\n', flush=True)
    args = ArgsParser.rpf_corr_args_parser()

    print('\nStep2: Import the RPFs file.\n', flush=True)
    rpf, sample_name, frame_rpm, frame0_rpm, frame1_rpm, frame2_rpm = read_rpf(args.rpf)

    print('\nStep3: calculate the RPM.\n', flush=True)
    gene_corr_f, gene_corr_f0, gene_corr_f1, gene_corr_f2 = calc_gene_rpm(rpf,
                                                                          sample_name, frame_rpm, frame0_rpm,
                                                                          frame1_rpm, frame2_rpm)
    rpf_corr_f, rpf_corr_f0, rpf_corr_f1, rpf_corr_f2 = calc_rpf_rpm(frame_rpm,
                                                                     frame0_rpm, frame1_rpm, frame2_rpm)

    print('\nStep4: Draw the correlation plot.\n', flush=True)
    draw_corr_plot(gene_corr_f, gene_corr_f0, gene_corr_f1, gene_corr_f2, 'gene', args.output)
    draw_corr_plot(rpf_corr_f, rpf_corr_f0, rpf_corr_f1, rpf_corr_f2, 'rpf', args.output)

    print('\nAll done.\n', flush=True)
    ArgsParser.now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Script  : fq2fa.py
# @Time    : 2020/7/10 16:00
# @project : riboparser
# @Version : python 3.7
# @Author  : Rensc
# @E-mail  : rensc0718@163.com


import argparse
import gc
import time
from argparse import RawTextHelpFormatter


# get arguments and define the scripts usage
def parse_args():
    parser = argparse.ArgumentParser(
        description="This script is used to convert the FQ to FA format.",
        formatter_class=RawTextHelpFormatter,
    )
    parser.add_argument("-v", "--version", action="version", version="%(prog)s 1.0")

    required = parser.add_argument_group(title="Required parameters")
    required.add_argument(
        "-i", metavar="input", required=True, type=str, help="input FQ sequence file"
    )
    required.add_argument(
        "-n", action='store_true',
        help="assign new id number for reads"
    )
    required.add_argument(
        "-o", metavar="out",  required=True, type=str,
        help="output file in FA format",
    )

    args = parser.parse_args()
    return args


# read the fastq file
def fq2fa(fq, new_id, fa):

    flag = 1
    with open(fq, "r") as inputs:
        with open(fa, "w") as outputs:
            if new_id:
                for line in inputs:
                    outputs.writelines(''.join(['>t', str(flag), '\n']))
                    outputs.writelines(''.join(inputs.__next__()))
                    inputs.__next__()
                    inputs.__next__()
                    flag += 1
            else:
                for line in inputs:
                    outputs.writelines(''.join(['>', line]))
                    outputs.writelines(''.join(inputs.__next__()))
                    inputs.__next__()
                    inputs.__next__()


# gc collect
def clear():
    print("Clear Memory.")
    gc.collect()
    print("All done.")


# get time
def now_time():
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))


# main
def main():
    now_time()
    args = parse_args()
    fq2fa(args.i, args.n, args.o)
    clear()
    now_time()


# run
if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_offset.py


import os
import pandas as pd
from collections import OrderedDict
import argparse


def merge_offset_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the reads offset files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for rsbm/sscbm offset files (e.g., '*rsbm_offset.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='prefix of output file name (default: prefix + _offset.txt).'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def process_offset_files(offset_list):
    '''
    @Message  : retrieve the offset table.
    @Input    : list --> pattern for reads offset files (e.g., '*rsbm_offset.txt')
    @Return   : 
                output --> output dataframe contain the reads offset
    @Flow     : step1 --> retrieve the reads offset
    '''

    # make the dataframe
    offset_colunms = ['Sample', 'Model', 'Length',
                      'Frame0', 'Rpfs0', 'Frame1', 'Rpfs1', 'Frame2', 'Rpfs2',
                      'P_site', 'Rpfs', 'Periodicity', 'Ribo']

    offset_merge = pd.DataFrame(columns=offset_colunms)

    # for each gne file
    for offset_file in offset_list:
        file_name = os.path.basename(offset_file).split('.')[0]

        if file_name.endswith('_rsbm_offset'):
            offset_model = 'RSBM'
        elif file_name.endswith('_sscbm_offset'):
            offset_model = 'SSCBM'
        else:
            print('Error: the file name is not correct, please check it.')
            exit()

        # import the offset file
        file_prefix = file_name.replace('_rsbm_offset', '').replace('_sscbm_offset', '')
        offset_df = pd.read_csv(offset_file, sep='\t', index_col=False)

        # convert the first chr of columns to upper
        offset_df.columns = [i[0].upper() + i[1:] for i in offset_df.columns]
        
        offset_df['Model'] = offset_model
        offset_df['Sample'] = file_prefix
        offset_df = offset_df[offset_colunms]

        # merge the gene offset file
        offset_merge = pd.concat([offset_merge, offset_df], axis=0)

    return offset_merge


def output_table(offset_merge, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> nested dict contain the reads offset
    @Return   : output --> description
    @Flow     : step1 --> convert the dict to dataframe
    '''

    # keep the 2 decimal places
    int_columns = ['Length', 'Frame0', 'Rpfs0', 'Frame1', 'Rpfs1', 'Frame2', 'Rpfs2', 'P_site', 'Rpfs']
    offset_merge[int_columns] = offset_merge[int_columns].astype('int')

    offset_merge.to_csv(output_file + '_offset.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_offset_args_parser()

    print('Step2: import the reads offset file.', flush=True)
    offset_merge = process_offset_files(args.list)

    print('Step3: output the reads offset table.', flush=True)
    output_table(offset_merge, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_quant.py


import pandas as pd
import argparse


def merge_quant_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the quant files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for quant files (e.g., '*_rpf_quant.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='prefix of output file name (default: prefix + _rpf_quant.txt).'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def import_quant_files(quant_list):
    '''
    @Message  : retrieve the quant table.
    @Input    : list --> pattern for quant files (e.g., '*_rpf_quant.txt')
    @Return   : 
                output --> output dataframe contain the quant
    @Flow     : step1 --> retrieve the quant
    '''

    quant_df = None
    # for each gne file
    for quant_file in quant_list:

        # import the quant file
        quant = pd.read_csv(quant_file, sep='\t', index_col=False)

        # merge the gene quant file
        if quant_df is None:
            quant_df = quant
        else:
            quant_df = pd.merge(quant_df, quant, on='name', how='outer')

    return quant_df


def output_table(quant_df, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> nested dict contain the quant
    @Return   : output --> description
    @Flow     : step1 --> convert the dict to dataframe
    '''

    quant_df.to_csv(output_file + '_rpf_quant.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_quant_args_parser()

    print('Step2: import the quant file.', flush=True)
    quant_df = import_quant_files(args.list)

    print('Step3: output the quant table.', flush=True)
    output_table(quant_df, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_period.py


import os
import pandas as pd
from collections import OrderedDict
import argparse


def merge_period_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the 3-nt periodicity files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for periodicity files (e.g., '*_periodicity.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='prefix of output file name (default: prefix + _periodicity.txt).'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def import_period_files(period_list):
    '''
    @Message  : retrieve the periodicity table.
    @Input    : list --> pattern for 3-nt periodicity files (e.g., '*_periodicity.txt')
    @Return   : 
                output --> output dataframe contain the 3-nt periodicity
    @Flow     : step1 --> retrieve the 3-nt periodicity
    '''

    # make the dataframe
    period_colunms = ['Sample', 'Frame', 'Count', 'Ratio']
    period_df = pd.DataFrame(columns=period_colunms)

    # for each gne file
    for period_file in period_list:
        # import the motif file
        period = pd.read_csv(period_file, sep='\t', index_col=False)

        # merge the gene periodion file
        period_df = pd.concat([period_df, period], axis=0)

    return period_df


def output_table(period_merge, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> nested dict contain the 3-nt periodicity
    @Return   : output --> description
    @Flow     : step1 --> convert the dict to dataframe
    '''

    period_merge.to_csv(output_file + '_periodicity.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_period_args_parser()

    print('Step2: import the 3-nt periodicity file.', flush=True)
    period_df = import_period_files(args.list)

    print('Step3: output the 3-nt periodicity table.', flush=True)
    output_table(period_df, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @time    : 2021/11/3 9:50
# @Project : riboParser
# @Script  : Properties.py


import sys
from collections import OrderedDict

import numpy as np
import pandas as pd

from Bio import SeqIO
from Bio.SeqUtils import ProtParam


class SeqProperties(object):

    def __init__(self, seq):
        '''
        @Message  : function for sequence properties calculation
        @Input    : param --> properties
        @Return   : output --> sequence propeerties
        @Flow     : step1 --> run ProteinAnalysis
        '''
        
        self.prpt = ProtParam.ProteinAnalysis(seq)
        self.gravy = str(round(self.prpt.gravy(), 3))
        if self.prpt.flexibility():
            self.flexibility = str(round(sum(self.prpt.flexibility()) / len(self.prpt.flexibility()), 3))
        else:
            self.flexibility = 'nan'
        self.instability = str(round(self.prpt.instability_index(), 3))
        self.isoelectric_point = str(round(self.prpt.isoelectric_point(), 3))
        self.structure = self.prpt.secondary_structure_fraction()
        self.helix = str(round(self.structure[0], 3))
        self.turn = str(round(self.structure[1], 3))
        self.sheet = str(round(self.structure[2], 3))


class Sequence(object):
    def __init__(self, args):
        self.fasta = args.fasta

        self.codon_dict = {
            "AAA": ["Lys", "K"],
            "AAC": ["Asn", "N"],
            "AAG": ["Lys", "K"],
            "AAT": ["Asn", "N"],
            "ACA": ["Thr", "T"],
            "ACC": ["Thr", "T"],
            "ACG": ["Thr", "T"],
            "ACT": ["Thr", "T"],
            "AGA": ["Arg", "R"],
            "AGC": ["Ser", "S"],
            "AGG": ["Arg", "R"],
            "AGT": ["Ser", "S"],
            "ATA": ["Ile", "I"],
            "ATC": ["Ile", "I"],
            "ATG": ["Met", "M"],
            "ATT": ["Ile", "I"],
            "CAA": ["Gln", "Q"],
            "CAC": ["HIS", "H"],
            "CAG": ["Gln", "Q"],
            "CAT": ["HIS", "H"],
            "CCA": ["Pro", "P"],
            "CCC": ["Pro", "P"],
            "CCG": ["Pro", "P"],
            "CCT": ["Pro", "P"],
            "CGA": ["Arg", "R"],
            "CGC": ["Arg", "R"],
            "CGG": ["Arg", "R"],
            "CGT": ["Arg", "R"],
            "CTA": ["Leu", "L"],
            "CTC": ["Leu", "L"],
            "CTG": ["Leu", "L"],
            "CTT": ["Leu", "L"],
            "GAA": ["Glu", "E"],
            "GAC": ["Asp", "D"],
            "GAG": ["Glu", "E"],
            "GAT": ["Asp", "D"],
            "GCA": ["Ala", "A"],
            "GCC": ["Ala", "A"],
            "GCG": ["Ala", "A"],
            "GCT": ["Ala", "A"],
            "GGA": ["Gly", "G"],
            "GGC": ["Gly", "G"],
            "GGG": ["Gly", "G"],
            "GGT": ["Gly", "G"],
            "GTA": ["Val", "V"],
            "GTC": ["Val", "V"],
            "GTG": ["Val", "V"],
            "GTT": ["Val", "V"],
            "TAA": ["Stop", "*"],
            "TAC": ["Tyr", "Y"],
            "TAG": ["Stop", "*"],
            "TAT": ["Tyr", "Y"],
            "TCA": ["Ser", "S"],
            "TCC": ["Ser", "S"],
            "TCG": ["Ser", "S"],
            "TCT": ["Ser", "S"],
            "TGA": ["Stop", "*"],
            "TGC": ["Cys", "C"],
            "TGG": ["Trp", "W"],
            "TGT": ["Cys", "C"],
            "TTA": ["Leu", "L"],
            "TTC": ["Phe", "F"],
            "TTG": ["Leu", "L"],
            "TTT": ["Phe", "F"],
        }
        
        self.codon_anno = pd.DataFrame.from_dict(self.codon_dict).T
        self.codon_anno = self.codon_anno.reset_index()
        self.codon_anno.columns = ['Codon', 'AA', 'Abbr.']

        self.cds_data_frame = ''

        # opts for file output
        self.output_prefix = args.output
        self.output_gene_cu = self.output_prefix + '.Gene.CodonUsage.xlsx'
        self.output_whole_cu = self.output_prefix + '.Whole.CodonUsage.txt'
        self.output_seq = self.output_prefix + '.Properties.txt'


    def count_codons_numpy(self, cds_sequence):
        '''
        @Message  : function for codon count summary.
        @Input    : param --> cds_sequence
        @Return   : output --> codon and count
        @Flow     : step1 --> split the cds sequence to codon sequence
                    step2 --> summary the codon count
        '''
        
        # ensure the length of the sequence is a multiple of 3
        if len(cds_sequence) % 3 != 0:
            raise ValueError("CDS length is not a multiple of 3")
        
        # split the sequence into codons
        codons = np.array([cds_sequence[i:i+3] for i in range(0, len(cds_sequence), 3)])
        unique, counts = np.unique(codons, return_counts=True)
        return dict(zip(unique, counts))
    
    def create_codon_table(self):
        '''
        @Message  : create the codon and amino acid table.
        @Input    : param --> self.fasta
        @Return   : output --> self.cds_data_frame
        @Flow     : step1 --> summary the codon and count of cds sequence
                    step2 --> merge all gene codon count to one data table
        '''
        
        codon_dict = OrderedDict()

        for record in SeqIO.parse(self.fasta, "fasta"):
            # convert the lower case to upper case
            record.seq = record.seq.upper()
            codon_count = self.count_codons_numpy(str(record.seq))
            codon_count['Length'] = len(record.seq) / 3
            codon_dict[record.id] = codon_count

        # convert codon count to codon table
        codon_table = pd.DataFrame(codon_dict).T.fillna(0)
        codon_table = codon_table.reset_index(names=['Gene'])

        # convert codon table to long format
        self.cds_data_frame = pd.melt(codon_table, id_vars=['Gene', 'Length'], 
                              value_vars=codon_table.columns.difference(['Gene', 'Length']),
                              var_name='Codon', value_name='Count')
        
        # merge codon annotation and sort the table
        self.cds_data_frame = self.cds_data_frame.merge(self.codon_anno, on='Codon', how='left')
        self.cds_data_frame = self.cds_data_frame.sort_values(by=['Gene', 'Abbr.']).reset_index(drop=True)


    def calc_gene_codon_usage(self):
        '''
        @Message  : function for gene codon usage calculation
        @Input    : param --> self.cds_data_frame
        @Return   : output --> Frequency, RSCU, CAI
        @Flow     : step1 --> calculate the codon frequency
                    step2 --> calculate the RSCU and CAI
                    step3 --> fill NA with 0 and round the value
                    step4 --> convert the data frame to wide format
        '''
        
        # calculate codon frequency
        self.cds_data_frame['Frequency'] = self.cds_data_frame['Count'] / self.cds_data_frame['Length'] * 1000

        # calculate RSCU and CAI
        self.cds_data_frame['RSCU'] = self.cds_data_frame.groupby(['Gene', 'AA'])['Frequency'].transform(lambda x: x / x.mean())
        self.cds_data_frame['CAI'] = self.cds_data_frame.groupby(['Gene', 'AA'])['Frequency'].transform(lambda x: x / x.max())

        # fill NA with 0
        self.cds_data_frame.fillna({'Frequency': 0, 'RSCU': 0, 'CAI': 0}, inplace=True)

        # round the value
        self.cds_data_frame['Frequency'] = self.cds_data_frame['Frequency'].round(4)
        self.cds_data_frame['RSCU'] = self.cds_data_frame['RSCU'].round(4)
        self.cds_data_frame['CAI'] = self.cds_data_frame['CAI'].round(4)

        # convert the data frame to wide format
        cds_freq_data_frame_wide = self.cds_data_frame.pivot_table(index=['Gene', 'Length'], columns='Codon', values='Frequency').reset_index()
        cds_rscu_data_frame_wide = self.cds_data_frame.pivot_table(index=['Gene', 'Length'], columns='Codon', values='RSCU').reset_index()
        cds_cai_data_frame_wide = self.cds_data_frame.pivot_table(index=['Gene', 'Length'], columns='Codon', values='CAI').reset_index()

        # output the three data frame
        cds_freq_data_frame_wide.to_csv(self.output_prefix + '_frequency.txt', sep='\t', index=False)
        cds_rscu_data_frame_wide.to_csv(self.output_prefix + '_rscu.txt', sep='\t', index=False)
        cds_cai_data_frame_wide.to_csv(self.output_prefix + '_cai.txt', sep='\t', index=False)

        # with pd.ExcelWriter(self.output_gene_cu) as writer:
        #     cds_freq_data_frame_wide.to_excel(writer, sheet_name='Frequency', index=False)
        #     cds_rscu_data_frame_wide.to_excel(writer, sheet_name='RSCU', index=False)
        #     cds_cai_data_frame_wide.to_excel(writer, sheet_name='CAI', index=False)

    def calc_whole_codon_usage(self):
        '''
        @Message  : function for whole codon usage calculation
        @Input    : param --> self.cds_data_frame
        @Return   : output --> Frequency, RSCU, CAI
        @Flow     : step1 --> calculate the codon frequency
                    step2 --> calculate the RSCU and CAI
                    step3 --> fill NA with 0 and round the value
                    step4 --> save the data frame to file
        '''
        
        cds_data_frame_sum = self.cds_data_frame.groupby(['Codon', 'AA', 'Abbr.'])['Count'].sum().reset_index()
        cds_data_frame_sum['Frequency'] = cds_data_frame_sum['Count'] / cds_data_frame_sum['Count'].sum() * 1000
        cds_data_frame_sum['RSCU'] = cds_data_frame_sum['Frequency'] / cds_data_frame_sum.groupby('AA')['Frequency'].transform('mean')
        cds_data_frame_sum['CAI'] = cds_data_frame_sum['Frequency'] / cds_data_frame_sum.groupby('AA')['Frequency'].transform('max')
        
        cds_data_frame_sum.fillna({'Frequency': 0, 'RSCU': 0, 'CAI': 0}, inplace=True)

        cds_data_frame_sum['Frequency'] = cds_data_frame_sum['Frequency'].round(4)
        cds_data_frame_sum['RSCU'] = cds_data_frame_sum['RSCU'].round(4)
        cds_data_frame_sum['CAI'] = cds_data_frame_sum['CAI'].round(4)

        cds_data_frame_sum.to_csv(self.output_prefix + '_whole_codon_usage.txt', sep='\t', index=False)

    def protein_analysis(self):
        '''
        @Message  : function for protein properties calculation
        @Input    : param --> self.fasta
        @Return   : output --> sequence properties
        @Flow     : step1 --> read the fasta file
                    step2 --> write the sequence properties to file
        '''
        
        with open(self.output_seq, 'w') as out_seq:
            out_seq.writelines('\t'.join(['ID', 'Seq', 'Length', 'Gravy', 'Flexibility', 'Instability',
                                          'Isoelectric_Point', 'Helix', 'Turn', 'Sheet']) + '\n')
            fa_seq = SeqIO.to_dict(SeqIO.parse(self.fasta, "fasta"))

            for now_idx, now_seq in fa_seq.items():
                now_aa = str(now_seq.seq.translate())
                now_len = str(len(now_aa))

                if now_aa[-1] == '*':
                    now_aa = now_aa[:-1]
                    now_len = str(len(now_aa))

                if '*' in now_aa:
                    sys.stdout.writelines(
                        'Skip sequence contain stop codon: {id}\t{aa}'.format(id=now_idx, aa=now_aa) + '\n')
                    continue

                prpt = SeqProperties(now_aa)
                out_seq.writelines('\t'.join([now_idx, now_aa, now_len, prpt.gravy, prpt.flexibility, prpt.instability,
                                              prpt.isoelectric_point, prpt.helix, prpt.turn, prpt.sheet]) + '\n')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : rpf_geneplot.py


import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns

from .ribo import ArgsParser


def read_gene_list(gene_list):
    gene_csv = pd.read_csv(gene_list, header=None, names=None)
    gene_name = []
    for gene in gene_csv.values.tolist():
        gene_name.extend(gene)

    return gene_name


def read_rpf(rpf_file, gene_name):
    if isinstance(gene_name, str):
        gene_name = [gene_name]
    else:
        pass

    rpf = pd.read_csv(rpf_file, sep='\t', header=0, names=None)

    sample_num = int((len(rpf.columns) - 6) / 3)
    sample_name = pd.Series(rpf.columns[6::]).str[:-3].drop_duplicates().tolist()
    total_cds_num = len(rpf["region"] == "cds") * 3

    total_rpf = []
    for num in range(sample_num):
        total_rpf.append(rpf.iloc[:, [6 + num * 3, 7 + num * 3, 8 + num * 3]].sum().sum())

    gene_rpf = rpf[rpf["name"].isin(gene_name)]
    merged_rpf = pd.DataFrame()

    # flt_gene_rpf1.drop(flt_gene_rpf1.columns[[1, 3, 4, 5]], axis=1, inplace=True)

    for gene in gene_name:
        now_gene_rpf = gene_rpf[gene_rpf["name"] == gene]
        if now_gene_rpf.empty:
            print('{0} are not fount in RPFs file.'.format(gene), flush=True)
            continue
        merged_gene_rpf = pd.DataFrame()

        for num in range(sample_num):
            now_idx = [0, 2, 3] + [6 + num * 3, 7 + num * 3, 8 + num * 3]
            flt_gene_rpf = now_gene_rpf.iloc[:, now_idx]
            melt_gene_rpf = pd.melt(flt_gene_rpf, id_vars=["name", "from_tis", "from_tts"]).sort_values(
                by=["name", "from_tis", "from_tts", "variable"])
            melt_gene_rpf = melt_gene_rpf.reset_index(drop=True)
            del melt_gene_rpf["variable"]
            melt_gene_rpf.columns = ['name', 'from_tis', 'from_tts', sample_name[num]]

            if not merged_gene_rpf.empty:
                merged_gene_rpf = pd.concat([merged_gene_rpf, melt_gene_rpf.iloc[:, -1]], axis=1)
            else:
                merged_gene_rpf = melt_gene_rpf.copy()

        if not merged_rpf.empty:
            merged_rpf = pd.concat([merged_rpf, merged_gene_rpf])
        else:
            merged_rpf = merged_gene_rpf.copy()

    total_rpf = pd.DataFrame(total_rpf).T

    merged_rpm = merged_rpf.iloc[:, 0:3].copy()
    merged_dst = merged_rpf.iloc[:, 0:3].copy()

    merged_rpm[sample_name] = merged_rpf[sample_name].div(total_rpf.values) * 1e7
    merged_dst[sample_name] = merged_rpf[sample_name].div(total_rpf.values) * total_cds_num

    return sample_num, sample_name, merged_rpm, merged_dst


def re_build_list(now_gene_rpm):
    start_site, stop_site = now_gene_rpm["from_tis"].iloc[0], now_gene_rpm["from_tis"].iloc[-1]
    start_nt_site = start_site * 3
    stop_nt_site = stop_site * 3 + 2

    codon = stop_site - start_site
    cds_length = codon * 3 + 2

    mrna_index = [i for i in range(start_nt_site, stop_nt_site + 1)]
    now_gene_rpm.index = mrna_index

    stop_codon_site = now_gene_rpm[now_gene_rpm["from_tts"] == 0]["from_tis"].index.tolist()[-1]
    now_labels = [start_nt_site, 0, stop_codon_site, stop_nt_site]

    return now_labels, cds_length, now_gene_rpm


def draw_density_plot(log_type, sample_num, sample_name, merged_rpm, merged_dst, gene_name):
    if isinstance(gene_name, str):
        gene_name = [gene_name]
    else:
        pass

    for gene in gene_name:
        now_gene_rpm = merged_rpm[merged_rpm["name"] == gene]
        if now_gene_rpm.empty:
            continue
        print("Now gene: {gene}.".format(gene=str(gene)), flush=True)
        now_labels, cds_length, draw_gene_rpm = re_build_list(now_gene_rpm)

        out_pdf = str(gene) + "_density_plot.pdf"
        out_png = str(gene) + "_density_plot.png"

        # matplotlib.use('AGG')

        # draw the periodicity of codon
        fig = plt.figure(figsize=(9, 3 * sample_num), dpi=300)

        for fig_num in range(sample_num):
            ax1 = fig.add_subplot(sample_num, 1, fig_num + 1)
            ax1.fill_between(draw_gene_rpm.index, draw_gene_rpm[sample_name[fig_num]], color="#f86934")
            # sns.barplot(x=draw_gene_rpm.index, y=draw_gene_rpm[sample_name[fig_num]], color="#f86934")
            # sns.lineplot(x=draw_gene_rpm.index, y=draw_gene_rpm[sample_name[fig_num]], color="#f86934")
            sns.lineplot(x=draw_gene_rpm.index, y=0, color="w")
            ax1.set_xticks(now_labels)
            ax1.set_xticklabels(now_labels)
            ax1.set_xlim([now_labels[0], now_labels[-1]])
            # ax1.set_ylim([0, int(now_gene_rpm.max() * 1.1)])

            # add the gene annotation here, need to import the txt file
            ax1.set_ylabel('RPTM')
            ax1.set_xlabel('position (nt)')
            ax1.set_title(sample_name[fig_num])
            plt.xticks(rotation=90)

            if log_type == "Not":
                pass
            else:
                plt.yscale('log', base=int(log_type))

        plt.suptitle(str(gene))
        plt.tight_layout()
        plt.show()

        fig.savefig(fname=out_pdf)
        fig.savefig(fname=out_png)
        plt.close()


def main():
    ArgsParser.now_time()
    print('\nDraw the gene RPFs plot.', flush=True)
    print('\nStep1: Checking the input Arguments.', flush=True)
    args = ArgsParser.rpf_gene_plot_args_parser()

    print('\nStep2: Import the gene name.', flush=True)

    if args.list:
        gene_name = read_gene_list(args.list)
    else:
        gene_name = args.gene

    print('\nStep3: Import the RPFs file.', flush=True)
    sample_num, sample_name, merged_rpm, merged_dst = read_rpf(args.rpf, gene_name)

    print('\nStep4: Draw the gene RPFs plot.', flush=True)
    draw_density_plot(args.log, sample_num, sample_name, merged_rpm, merged_dst, gene_name)

    print('\nAll done.', flush=True)
    ArgsParser.now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : phred_quality.py


import sys
import argparse
import gc
import gzip

scType = [
    "Unknown data score format!",
    "Sanger | Phred+33 | Qual[33,73] | Val(0,40)",
    "Solexa | Solexa+64 | Qual[59,104] | Val(-5,40)",
    "Illumina1.3 | Phred+64 | Qual[64,104] | Val(0,40)",
    "Illumina1.5 | Phred+64 | Qual[66,104] | Val(3,40)",
    "Illumina1.8 | Phred+33 | Qual[33,74] | Val(0,41)"
]

def readFile(fq_read, result_list, num):
    flag = 1
    for line in fq_read:
        if flag > num * 4:
            break
        elif flag % 4 == 0:
            result_list.append(line.strip())
        flag += 1

def judgeFile(input_fq, num, result_list):
    basename = str(input_fq).split('.')[-1]
    if basename in ["fq", "fastq"]:
        with open(input_fq, 'r') as fq_read:
            readFile(fq_read, result_list, num)
    elif basename == "gz":
        with gzip.open(input_fq, 'rb') as fq_read:
            readFile(fq_read, result_list, num)
    else:
        print("Please input the FASTQ file!")
        sys.exit()

def checkFq(input_fq, result_list):
    score = []
    for line in result_list:
        for ABC in line:
            score.extend([str(ord(ABC))])
    MIN = int(min(score))
    MAX = int(max(score))

    if MIN < 33 or MAX > 104:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[0]))
    elif MIN >= 33 and MAX <= 73:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[1]))
    elif MIN >= 59 and MAX <= 104:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[2]))
    elif MIN >= 64 and MAX <= 104:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[3]))
    elif MIN >= 66 and MAX <= 104:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[4]))
    elif MIN >= 33 and MAX <= 74:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[5]))
    else:
        print ("%s [%s,%s] ==> %s" % (input_fq, MIN, MAX, scType[0]))


def args_parser():
    parser = argparse.ArgumentParser(description='This script is used to check the quality scores of FASTQ files.')
    parser.add_argument('-i', '--input', required=True, help='Specify the input FASTQ file (gzip format is OK).')
    parser.add_argument('-n', '--number', type=int, default=40000, help='Specify the number of sequences to extract (default: 40000).')
    args = parser.parse_args()
    return args

def main():
    args = args_parser()
    input_fq = args.input
    num = args.number
    result_list = []

    judgeFile(input_fq, num, result_list)
    checkFq(input_fq, result_list)
    gc.collect()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Script  : nt2aa.py
# @Version : python 3.8.5
# @E-mail  : rensc0718@163.com


import argparse
import gc
import re
import sys
import time
from collections import OrderedDict


# get arguments and define the scripts usage
def parse_args():
    parser = argparse.ArgumentParser(description="This script is used to convert the nucleotide to amino acid.")
    parser.add_argument(
        "-v", "--version", action="version", version="%(prog)s 1.0"
    )
    parser.add_argument(
        "-i", "--input", required=True, type=str, help="input cds sequences in fasta format"
    )
    parser.add_argument(
        "-t", "--tis", required=False, action='store_true', default=False,
        help="start with ATG (default %(default)s)"
    )
    parser.add_argument(
        "-o", "--output", required=False, type=str, default='results-aa.fa',
        help="output file name (default %(default)s)"
    )

    args = parser.parse_args()

    args_dict = vars(args)
    for k, v in args_dict.items():
        print(k + ": " + str(v))
    print()
    sys.stdout.flush()
    return args


codon_dict = {'AAA': 'K', 'AAC': 'N', 'AAG': 'K', 'AAT': 'N', 'ACA': 'T', 'ACC': 'T',
              'ACG': 'T', 'ACT': 'T', 'AGA': 'R', 'AGC': 'S', 'AGG': 'R', 'AGT': 'S',
              'ATA': 'I', 'ATC': 'I', 'ATG': 'M', 'ATT': 'I', 'CAA': 'Q', 'CAC': 'H',
              'CAG': 'Q', 'CAT': 'H', 'CCA': 'P', 'CCC': 'P', 'CCG': 'P', 'CCT': 'P',
              'CGA': 'R', 'CGC': 'R', 'CGG': 'R', 'CGT': 'R', 'CTA': 'L', 'CTC': 'L',
              'CTG': 'L', 'CTT': 'L', 'GAA': 'E', 'GAC': 'D', 'GAG': 'E', 'GAT': 'D',
              'GCA': 'A', 'GCC': 'A', 'GCG': 'A', 'GCT': 'A', 'GGA': 'G', 'GGC': 'G',
              'GGG': 'G', 'GGT': 'G', 'GTA': 'V', 'GTC': 'V', 'GTG': 'V', 'GTT': 'V',
              'TAA': '*', 'TAC': 'Y', 'TAG': '*', 'TAT': 'Y', 'TCA': 'S', 'TCC': 'S',
              'TCG': 'S', 'TCT': 'S', 'TGA': '*', 'TGC': 'C', 'TGG': 'W', 'TGT': 'C',
              'TTA': 'L', 'TTC': 'F', 'TTG': 'L', 'TTT': 'F'}


def nt2aa(nt_file_in, aa_file_out):
    gene = OrderedDict()
    with open(nt_file_in, 'r') as nt_file:
        for line in nt_file:

            if line.startswith('>'):
                gene_name = line.strip()
                gene[gene_name] = ''
            else:
                rec = line.strip().upper().replace('U', 'T')
                gene[gene_name] += rec

    with open(aa_file_out, 'w') as aa_file:
        for gene_name, seq in gene.items():
            aa = ''
            unexp_nt = re.sub(r'[ATGCU]', '', seq)

            if len(seq) % 3 != 0:
                print("length error:\t" + gene_name + '\t' + str(len(seq)))
                continue
            elif len(unexp_nt) > 0:
                print('nucleotide error:\t' + gene_name + '\t' + ''.join(set(list(unexp_nt))))
                continue
            else:
                for num in range(0, len(seq), 3):
                    nt = seq[num:num + 3]
                    aa += codon_dict[nt]
                aa_file.writelines('\n'.join([gene_name, aa]) + '\n')


def nt2aa_atg(nt_file_in, aa_file_out):
    gene = OrderedDict()
    with open(nt_file_in, 'r') as nt_file:
        for line in nt_file:

            if line.startswith('>'):
                gene_name = line.strip()
                gene[gene_name] = ''
            else:
                rec = line.strip().upper().replace('U', 'T')
                gene[gene_name] += rec

    with open(aa_file_out, 'w') as aa_file:
        for gene_name, seq in gene.items():
            aa = ''
            unexp_nt = re.sub(r'[ATGCU]', '', seq)
            stop_codon = seq[-3::]

            if len(seq) % 3 != 0:
                print("length error:\t" + gene_name + '\t' + str(len(seq)))
                continue
            elif not seq.startswith('ATG'):
                print('start codon error:\t' + gene_name + '\t' + seq[0:3])
                continue
            elif len(unexp_nt) > 0:
                print('nucleotide error:\t' + gene_name + '\t' + ''.join(set(list(unexp_nt))))
                continue
            elif stop_codon != 'TGA' and stop_codon != 'TAG' and stop_codon != 'TAA':
                print('stop codon warning:\t' + gene_name + '\t' + stop_codon)
                for num in range(0, len(seq), 3):
                    nt = seq[num:num + 3]
                    aa += codon_dict[nt]
                aa_file.writelines('\n'.join([gene_name, aa]) + '\n')
            else:
                for num in range(0, len(seq), 3):
                    nt = seq[num:num + 3]
                    aa += codon_dict[nt]
                aa_file.writelines('\n'.join([gene_name, aa]) + '\n')


# gc collect
def clear():
    gc.collect()
    print("\nAll done.")
    sys.stdout.flush()


# get time
def now_time():
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
    sys.stdout.flush()


# main
def main():
    args = parse_args()
    if args.tis:
        nt2aa_atg(args.input, args.output)
    else:
        nt2aa(args.input, args.output)
    clear()
    now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Project      : riboParser
@Script       : fq_trim.py
@Environment  : python 3.8.5
@Version      : 1.0
@Author       : Rensc 
@Time         : 2023/11/07 17:47:32
@E-mail       : rensc0718@163.com
@License      : (C)Copyright 2023-2025, Ren Shuchao
'''


# import pandas as pd
# import polars as pl
# import numpy as np
# from collections import OrderedDict
# from Bio import SeqIO
import argparse


def fq_trim_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to trim the fastq file.')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        '-i', dest='input', required=True, type=str, help='input fq file name.'
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='output fq file name.'
    )

    # optional arguments
    parser.add_argument(
        '-l', dest='length', required=False, type=int, default=30, 
        help='nucleotide length (default: %(default)s nt).'
    )
    parser.add_argument(
        '-d', dest='discard', required=False, action='store_true', default=False, 
        help='discard the reads less than minimum length (default: %(default)s).'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def trim_fq(args):
    '''
    @Message  : function for trim the fastq file.
    @Input    : fastq --> input file
                length --> length of nucleotide
    @Return   : output --> trimmed fastq file
    '''
    
    with open(args.input, 'r') as fq_in:
        with open(args.output, 'w') as fq_out:

            for line in fq_in:
                read_id = line.strip()
                read_seq = line.next().strip()
                read_name = line.next().strip()
                read_score = line.next().strip()

                if len(read_seq) >= args.length:
                    fq_out.writelines(read_id + '\n')
                    fq_out.writelines(read_seq[0:args.length] + '\n')
                    fq_out.writelines(read_name + '\n')
                    fq_out.writelines(read_score[0:args.length] + '\n')
                else:
                    pass


def main():
    args = fq_trim_args_parser()
    trim_fq(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# @Project : riboParser
# @Script  : dos2unix.py


import sys
import argparse


def args_parser():
    parser = argparse.ArgumentParser(description='This script is used to convert the dos format to unix format.')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument('-i', dest='input', required=True, type=str, help='input file name')
    input_group.add_argument('-o', dest='output', required=True, type=str, help='output file name')

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print("{:<12}:  {:<}".format(k, str(v)), flush=True)
    
    return args


def dos2unix(input_file, output_file):

    with open(output_file, 'w') as outfile:
        with open(input_file, 'r') as infile:
            for line in infile:
                line = line.strip()
                print(line)
                outfile.writelines(''.join([line, '\n']))


def main():
    print("Convert the dos file to unix format.", flush=True)
    args = args_parser()

    print("\nStep1: output the unix file.", flush=True)
    dos2unix(args.input, args.output)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : serp_properties.py


from .ribo import ArgsParser
from .serp import Properties


def main():
    ArgsParser.now_time()
    
    print('\nEvaluate the different properties of sequence.\n', flush=True)
    print('Step1: Checking the input Arguments.\n', flush=True)
    args = ArgsParser.serp_properties()

    print('\nStep2: Import the sequence.\n', flush=True)
    seq = Properties.Sequence(args)

    print('\nStep3: Calculate the gene codon usage.\n', flush=True)
    seq.create_codon_table()
    seq.calc_gene_codon_usage()

    print('\nStep4: Calculate the whole codon usage.\n', flush=True)
    seq.calc_whole_codon_usage()

    print('\nStep4: Calculate the properties of sequence.\n', flush=True)
    seq.protein_analysis()

    print('\nAll done.\n', flush=True)
    ArgsParser.now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_pausing.py


import pandas as pd
import argparse


def merge_pausing_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the pausing score files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for pausing score files (e.g., '*sum_codon_pausing_score.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='prefix of output file name (default: prefix + sum_codon_pausing_score.txt).'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def import_pausing_files(pausing_list):
    '''
    @Message  : retrieve the pausing table.
    @Input    : list --> pattern for pausing files (e.g., '*sum_codon_pausing_score.txt')
    @Return   : 
                output --> output dataframe contain the pausing
    @Flow     : step1 --> retrieve the pausing
    '''

    # make the dataframe
    pausing_df = pd.DataFrame()

    # for each pausing file
    for pausing_file in pausing_list:
        # import the pausing file
        pausing = pd.read_csv(pausing_file, sep='\t', index_col=False)
        pausing.sort_values(by=['Abbr', 'Codon'], ascending=True, inplace=True, ignore_index=True)

        # merge the gene pausing file
        if pausing_df.empty:
            pausing_df = pausing
        else:
            # delete the columns ['AA', 'Abbr.', 'count']
            pausing = pausing.drop(['AA', 'Abbr', 'Count'], axis=1)
            pausing_df = pd.merge(left=pausing_df, right=pausing, how='outer', on='Codon')

    # reorder the pausing columns with 
    # ['codon', 'AA', 'Abbr.', 'count', *_valid_codon, *_rpf_count, *_absolute_total_ps, *_absolute_valid_ps, *_relative_total_ps, *_relative_valid_ps]
    pausing_codon = pausing_df[['Codon', 'AA', 'Abbr', 'Count']]
    pausing_valid_codon = pausing_df.filter(regex='_valid_codon')
    pausing_rpf_count = pausing_df.filter(regex='_rpf_count')
    pausing_total_ps = pausing_df.filter(regex='_absolute_total_ps')
    pausing_valid_ps = pausing_df.filter(regex='_absolute_valid_ps')
    pausing_rel_total_ps = pausing_df.filter(regex='_relative_total_ps')
    pausing_rel_valid_ps = pausing_df.filter(regex='_relative_valid_ps')

    pausing_merge = pd.concat([pausing_codon, pausing_valid_codon, pausing_rpf_count,
                               pausing_total_ps, pausing_valid_ps, pausing_rel_total_ps, pausing_rel_valid_ps], 
                               axis=1)

    return pausing_merge


def output_table(pausing_merge, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> nested dict contain the pausing
    @Return   : output --> description
    @Flow     : step1 --> convert the dict to dataframe
    '''

    pausing_merge.to_csv(output_file + '_sum_codon_pausing_score.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_pausing_args_parser()

    print('Step2: import the pausing file.', flush=True)
    pausing_df = import_pausing_files(args.list)

    print('Step3: output the pausing table.', flush=True)
    output_table(pausing_df, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_offset_detail.py


import os
import pandas as pd
from collections import OrderedDict
import argparse


def merge_offset_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the details of offset files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for offset end site files (e.g., '*end.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='output file name.'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def import_offset_files(offset_list):
    '''
    @Message  : retrieve the offset table.
    @Input    : list --> pattern for reads offset files (e.g., '*end.txt')
    @Return   : 
                output --> output dataframe contain the reads offset
    @Flow     : step1 --> retrieve the reads offset
    '''

    offset_dict = OrderedDict()

    # for each gene file
    for offset_file in offset_list:
        file_name = os.path.basename(offset_file).split('.')[0]

        if file_name.endswith('tis_3end'):
            offset_model = 'tis_3end'
        elif file_name.endswith('tis_5end'):
            offset_model = 'tis_5end'
        elif file_name.endswith('tts_3end'):
            offset_model = 'tts_3end'
        elif file_name.endswith('tts_5end'):
            offset_model = 'tts_5end'
        else:
            print('Error: the file name is not correct, please check it.')
            exit()

        offset_site = offset_model.replace('_3end', '').replace('_5end', '').upper()
        # if offset_model.replace('tis_', '').replace('tts_', '') == '3end': offset_end = '3end', else: offset_end = '5end'
        offset_end = "3p-end" if offset_model.replace('tis_', '').replace('tts_', '') == '3end' else "5p-end"

        # import the offset file
        file_prefix = file_name.replace('_3end', '').replace('_5end', '').replace('_tis', '').replace('_tts', '')
        offset_df = pd.read_csv(offset_file, sep='\t', index_col=0)
        
        try:
            offset_dict[file_prefix][offset_model] = offset_df

        except KeyError:
            offset_dict[file_prefix] = {}
            offset_dict[file_prefix][offset_model] = offset_df
            
        offset_dict[file_prefix][offset_model].index.name = 'Length'
        offset_dict[file_prefix][offset_model].reset_index(inplace=True)
        offset_dict[file_prefix][offset_model].insert(0, 'End', offset_end)
        offset_dict[file_prefix][offset_model].insert(0, 'Site', offset_site)
        offset_dict[file_prefix][offset_model].insert(0, 'Sample', file_prefix)

        # convert the table too the long table
        offset_dict[file_prefix][offset_model] = offset_dict[file_prefix][offset_model].melt(id_vars=['Sample', 'Length', 'Site', 'End'], var_name='Offset', value_name='Count')


    return offset_dict


def process_offset_dict(offset_dict):
    '''
    @Message  : retrieve the offset table.
    @Input    : list --> pattern for reads offset files (e.g., '*end.txt')
    @Return   : 
                output --> output dataframe contain the reads offset
    @Flow     : step1 --> retrieve the reads offset
    '''

    offset_5end_merge = pd.DataFrame()
    offset_3end_merge = pd.DataFrame()

    # for each gene file
    for file, offset in offset_dict.items():
        
        # for each offset model
        offset_5end = pd.concat([offset['tis_5end'], offset['tts_5end']], axis=0).reset_index(drop=True)
        offset_3end = pd.concat([offset['tis_3end'], offset['tts_3end']], axis=0).reset_index(drop=True)

        # merge the gene offset file
        if offset_5end_merge.empty:
            offset_5end_merge = offset_5end
        else:
            offset_5end_merge = pd.concat([offset_5end_merge, offset_5end], axis=0)

        if offset_3end_merge.empty:
            offset_3end_merge = offset_3end
        else:
            offset_3end_merge = pd.concat([offset_3end_merge, offset_3end], axis=0)

    offset_3end_merge = offset_3end_merge.dropna(axis=1, how='all')
    offset_5end_merge = offset_5end_merge.dropna(axis=1, how='all')
    offset_merged = pd.concat([offset_5end_merge, offset_3end_merge], axis=0)

    return offset_merged



def output_table(offset_merged, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> dataframe contain the reads offset
    @Return   : output --> description
    @Flow     : step1 --> output the dataframe to txt file
    '''
    
    offset_merged.to_csv(output_file + '_offset_end.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_offset_args_parser()

    print('Step2: import the end of offset file.', flush=True)
    offset_dict = import_offset_files(args.list)
    offset_merged = process_offset_dict(offset_dict)

    print('Step3: output the end of offset table.', flush=True)
    output_table(offset_merged, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# @Project : RiboParser
# @Script  : rna_Offset.py


from .ribo import ArgsParser
import pandas as pd
import numpy as np


def main():
    ArgsParser.now_time()
    print('Create the p-site offset for RNA-seq.\n', flush=True)
    print('Step1: Checking the input Arguments.\n', flush=True)
    args = ArgsParser.rna_offset_args_parser()
    
    # create the Offset table
    min_length = args.min
    max_length = args.max
    length_range = np.arange(min_length, max_length + 1)
    offset_table = pd.DataFrame(length_range, index=length_range, columns=['length'])

    offset_table['frame0'] = args.exp_offset
    offset_table['rpfs0'] = 0

    offset_table['frame1'] = args.exp_offset + 1
    offset_table['rpfs1'] = 0
    
    offset_table['frame2'] = args.exp_offset + 2
    offset_table['rpfs2'] = 0

    offset_table['rpfs'] = 0

    offset_table['p_site'] = args.exp_offset + 1
    offset_table['periodicity'] = 100
    offset_table['ribo'] = 'first'

    # output the offset table
    print('Step2: Output the offset table.\n', flush=True)
    offset_table.to_csv(args.output + '_offset.txt', sep='\t', index=False)
    
    ArgsParser.now_time()
    print('All done.\n', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : rpf_meta_codon.py


from .ribo.ArgsParser import *
from .ribo.MetaCodon import *


def main():
    now_time()

    print('\nDraw the meta-codon plot.', flush=True)
    print('\nStep1: Checking the input Arguments.', flush=True)
    args = meta_codon_plot_args_parser()
    meta_codon = MetaCodon(args)

    print('\nStep2: Import the codon list.', flush=True)
    meta_codon.import_codon()

    print('\nStep3: Import the RPFs file.', flush=True)
    meta_codon.import_rpf()
    meta_codon.smooth_rpf_density()

    print('\nStep4: Retrieve specific codon density.', flush=True)
    meta_codon.reterieve_codon_density()

    print('\nStep5: Output the specific codon meta RPFs density.', flush=True)
    meta_codon.output_meta_codon_density()

    print('\nStep6: Output the specific codon sequence.', flush=True)
    meta_codon.output_meta_codon_seq()

    print('\nStep7: Draw the specific codon.', flush=True)
    meta_codon.draw_meta_codon()


    print('\nAll done.', flush=True)
    now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : detect_offset.py


from .ribo import ArgsParser
from .ribo.Offset import *


def main():
    ArgsParser.now_time()
    print('Detect the p-site offset.\n', flush=True)
    print('Step1: Checking the input Arguments.\n', flush=True)
    args = ArgsParser.offset_args_parser()
    offset_attr = Offset(args)

    print('Step2: Import the transcripts annotation.\n', flush=True)
    offset_attr.read_transcript()

    print('Step3: Import the bam file.\n', flush=True)
    offset_attr.get_mrna_reads()

    if args.mode == 'SSCBM':
        print('Step4: Detect the SSCBM offset of sequence profile.\n', flush=True)
        offset_attr.get_tis_offset()
        offset_attr.adjust_tis_offset()
        offset_attr.write_tis_offset()

        offset_attr.draw_tis_heatmap()

        print('Step5: Detect the RSBM offset of sequence profile.\n', flush=True)
        offset_attr.get_frame_offset()
        offset_attr.format_frame_offset()
        offset_attr.adjust_frame_offset()
        offset_attr.write_frame_offset()

        offset_attr.draw_frame_heatmap()

    elif args.mode == 'RSBM':
        print('Step4: Detect the RSBM offset of sequence profile.\n', flush=True)
        offset_attr.get_frame_offset()
        offset_attr.format_frame_offset()
        offset_attr.adjust_frame_offset()
        offset_attr.write_frame_offset()

        offset_attr.draw_frame_heatmap()

    ArgsParser.now_time()
    print('All done.\n', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#@Project : riboParser
#@Script  : merge_digestion.py


import os
import pandas as pd
from collections import OrderedDict
import argparse


def merge_digestion_args_parser():

    parser = argparse.ArgumentParser(description='This script is used to merge the reads digestion files')

    # needed arguments
    input_group = parser.add_argument_group('Required arguments')
    input_group.add_argument(
        "-l", "--list", nargs='+', required=True, help="List for digestion files (e.g., '*_5end_pwm.txt')."
    )
    input_group.add_argument(
        '-o', dest='output', required=True, type=str, help='prefix of output file name.'
    )

    args = parser.parse_args()
    args_dict = vars(args)
    for k, v in args_dict.items():
        print('{:<12}:  {:<}'.format(k, str(v)), flush=True)

    return args


def process_digest_files(digestion_list):
    '''
    @Message  : retrieve the pwm table.
    @Input    : list --> pattern for reads digestion files (e.g., '*_pwm.txt')
    @Return   : 
                output --> output dataframe contain the reads digestion
    @Flow     : step1 --> retrieve the reads digestion
    '''

    # make the dataframe
    digest_colunms = ['Sample', 'Digest', 'Site', 'A', 'T', 'C', 'G']
    digestion_df = pd.DataFrame(columns=digest_colunms)

    # for each gne file
    for motif_file in digestion_list:
        file_name = os.path.basename(motif_file).split('.')[0]

        if file_name.endswith('_5end_pwm'):
            Digest = 'End_5p'
        elif file_name.endswith('_3end_pwm'):
            Digest = 'End_3p'
        else:
            print('Error: the file name is not correct, please check it.')
            exit()

        file_prefix = file_name.replace('_5end_pwm', '').replace('_3end_pwm', '')

        # import the motif file
        motif_df = pd.read_csv(motif_file, sep='\t', index_col=0)

        # rename index to Site
        motif_df.index.name = 'Site'
        motif_df.reset_index(inplace=True)

        # set the site range of 5' end and 3' end
        if Digest == 'End_5p':
            motif_df['Site'] = motif_df['Site'] - 5
        elif Digest == 'End_3p':
            motif_df['Site'] = motif_df['Site'] - 10

        motif_df['Digest'] = Digest
        motif_df['Sample'] = file_prefix
        motif_df = motif_df[digest_colunms]

        # merge the gene digestion file
        digestion_df = pd.concat([digestion_df, motif_df], axis=0)

    return digestion_df


def output_table(digestion_df, output_file):
    '''
    @Message  : function for output.
    @Input    : result_dict --> nested dict contain the reads digestion motif
    @Return   : output --> description
    @Flow     : step1 --> convert the dict to dataframe
    '''

    # keep the 2 decimal places
    digestion_df['Site'] = digestion_df['Site'].astype('int')

    digestion_df.to_csv(output_file + '_reads_digestion.txt', sep='\t', index=False)


def main():

    print('Step1: Checking the input Arguments.', flush=True)
    args = merge_digestion_args_parser()

    print('Step2: improt the reads digestion file.', flush=True)
    digestion_df = process_digest_files(args.list)

    print('Step3: output the reads digestion table.', flush=True)
    output_table(digestion_df, args.output)

    print('All done.', flush=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @time    : 2020/12/1 20:15
# @Project : mytools
# @Script  : fa_split.py
# @Version : python 3.8.5
# @product : PyCharm
# @Author  : Rensc
# @E-mail  : rensc0718@163.com


import argparse
import gc
import time
import sys
from argparse import RawTextHelpFormatter


# get arguments and define the scripts usage
def parse_args():
    parser = argparse.ArgumentParser(description="This script is used to split the fasta files.")
    parser.add_argument(
        "-v", "--version", action="version", version="%(prog)s 1.1"
    )
    parser.add_argument(
        "-i", "--input", required=True, type=str, help="input cds sequences in fasta format"
    )
    parser.add_argument(
        "-n", "--num", required=False, type=int, default=10,
        help="number of files (default %(default)s)"
    )
    parser.add_argument(
        "-o", "--output", required=False, type=str, default='split',
        help="prefix of output file name (default %(default)s)"
    )

    args = parser.parse_args()

    args_dict = vars(args)
    for k, v in args_dict.items():
        print(k + ": " + str(v))

    sys.stdout.flush()
    return args


fa_list = []


# read the cdna sequence file
def read_fa(input_fa):
    with open(input_fa, 'r') as fa_read:
        ID, seq = '', ''
        for line1 in fa_read:
            if line1.startswith('>') and ID != '':
                fa_list.append(ID)
                fa_list.append(seq)
                ID = line1.strip()
                seq = ''
            elif line1.startswith('>'):
                ID = line1.strip()
            else:
                seq = seq + line1.strip()
        fa_list.append(ID)
        fa_list.append(seq)


def split_fa(num, output_fa):
    lines = len(fa_list)
    flag = 0

    for file_num in range(num):
        with open(output_fa + str(file_num + 1) + ".fa", 'w') as fa_write:
            for line_num in range(0, int(lines / num), 2):
                if flag < lines:
                    fa_write.writelines('\n'.join([fa_list[flag], fa_list[flag + 1]]) + '\n')
                    flag += 2
                else:
                    break


# gc collect
def clear():
    gc.collect()
    print("All done.")
    sys.stdout.flush()


# get time
def now_time():
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()))
    sys.stdout.flush()


# main
def main():
    now_time()
    args = parse_args()
    read_fa(args.input)
    if args.output:
        split_fa(args.num, args.output)
    else:
        output_fa = args.input.split('.')[0] + '-part'
        split_fa(args.num, output_fa)
    clear()
    now_time()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : Metaplot.py


from collections import OrderedDict
import matplotlib
import matplotlib.pyplot as plt
import polars as pl
import pandas as pd
import numpy as np


class Metaplot(object):

    def __init__(self, args):
        # input and output
        self.transcript = args.transcript
        self.gene = None
        self.rpf = args.rpf
        self.output = args.output

        # filter the range around tis/tts
        self.rpf_num = args.min
        self.utr5 = args.utr5
        self.cds = args.cds
        self.utr3 = args.utr3

        self.norm = args.normal
        self.sample_num = 0
        self.sample_name = None
        self.sample_dict = None

        self.tis_meta = None
        self.tts_meta = None
        self.merge_meta = pd.DataFrame()

        # for the high expression gene filter
        self.raw_rpf = None
        self.merged_rpf = None
        self.high_gene = None
        self.high_rpf = None
        self.gene_rpf_sum = None
        self.total_rpf_num = None

    def gene_anno(self):
        '''
        @Message  : function for gene annotation filter
        @Input    : self.utr5 --> input utr5 length
                    self.cds --> input cds length
                    self.utr3 --> input utr3 length
                    self.transcript --> input transcript file
        @Return   : self.gene --> output gene table
        @Flow     : step1 --> import the transcript file
                    step2 --> filter the transcript file with the input utr5/cds/utr3 length
                    step3 --> output the gene table
        '''
        
        anno = pd.read_csv(self.transcript, sep='\t', header=0, names=None)
        max_utr5 = anno['utr5_length'].max()
        max_cds = int(anno['cds_length'].max() / 2)
        max_utr3 = anno['utr3_length'].max()

        if self.utr5 > max_utr5:
            print('Specified UTR5 length more than max UTR5 length, median length instead.', flush=True)
            print('Specified UTR5 length: ' + str(self.utr5), flush=True)
            print('Max UTR5 length: ' + str(max_utr5), flush=True)
            print('Median UTR5 length: ' + str(self.utr5), flush=True)
            self.utr5 = int(anno['utr5_length'].median())
            anno_flt = anno.loc[anno['utr5_length'] >= self.utr5, ].copy()
        else:
            anno_flt = anno.loc[anno['utr5_length'] >= self.utr5, ].copy()

        if self.cds > max_cds:
            print('Specified CDS length more than half max CDS length, half median length instead.', flush=True)
            print('Specified CDS length: ' + str(self.cds), flush=True)
            print('Max CDS length: ' + str(max_utr5), flush=True)
            print('Median CDS length: ' + str(self.cds), flush=True)
            self.cds = int(anno['cds_length'].median() / 2)
            anno_flt = anno_flt.loc[anno['cds_length'] >= self.cds * 2, ].copy()
        else:
            anno_flt = anno_flt.loc[anno['cds_length'] >= self.cds * 2, ].copy()

        if self.utr3 > max_utr3:
            print('Specified UTR3 length more than max UTR3 length, median length instead.', flush=True)
            print('Specified UTR3 length: ' + str(self.utr3), flush=True)
            print('Max UTR3 length: ' + str(max_utr3), flush=True)
            print('Median UTR3 length: ' + str(self.utr3), flush=True)
            self.utr3 = int(anno['utr3_length'].median())
            anno_flt = anno_flt.loc[anno['utr3_length'] >= self.utr3, ].copy()
        else:
            anno_flt = anno_flt.loc[anno['utr3_length'] >= self.utr3, ].copy()

        self.gene = anno_flt['transcript_id']

    def output_meta(self, now_sp):
        out_txt = now_sp + "_tis_tts_metaplot.txt"

        # output the tis meta
        self.tis_meta = pd.melt(self.tis_meta, id_vars='Codon', var_name='Sample', value_name='Density')
        self.tis_meta['Frame'] = self.tis_meta['Sample'].str[-1].astype(int)
        self.tis_meta["Nucleotide"] = self.tis_meta["Codon"] * 3 + self.tis_meta["Frame"]
        self.tis_meta.sort_values(by=['Nucleotide', 'Frame'], inplace=True)
        self.tis_meta['Sample'] = now_sp
        self.tis_meta['Meta'] = "TIS"
        self.tis_meta = self.tis_meta.loc[:, ['Sample', 'Meta', 'Nucleotide', 'Codon', 'Frame', 'Density']]

        # output the tts meta
        self.tts_meta = pd.melt(self.tts_meta, id_vars='Codon', var_name='Sample', value_name='Density')
        self.tts_meta['Frame'] = self.tts_meta['Sample'].str[-1].astype(int)
        self.tts_meta["Nucleotide"] = self.tts_meta["Codon"] * 3 + self.tts_meta["Frame"]
        self.tts_meta.sort_values(by=['Nucleotide', 'Frame'], inplace=True)
        self.tts_meta['Sample'] = now_sp
        self.tts_meta['Meta'] = "TTS"
        self.tts_meta = self.tts_meta.loc[:, ['Sample', 'Meta', 'Nucleotide', 'Codon', 'Frame', 'Density']]

        tis_tts_meta = pd.concat([self.tis_meta, self.tts_meta], axis=0)
        tis_tts_meta.to_csv(out_txt, sep='\t', index=False)


    def output_merge_meta(self):
        # output the merge meta
        out_txt = self.output + "_tis_tts_metaplot.txt"
        self.merge_meta.to_csv(out_txt, sep='\t', index=False)


    def read_rpf(self):
        '''
        @Message  : function for read rpf file
        @Input    : self.rpf --> input rpf file
                    
        @Return   : output --> description
        @Flow     : step1 --> import the rpf file
                    step2 --> filter the rpf file with the input gene id
                    step3 --> output the rpf file
        '''
        
        self.raw_rpf = pl.read_csv(self.rpf, has_header = True, separator = '\t')
        self.raw_rpf = self.raw_rpf.to_pandas()

        rpf_mer = self.raw_rpf.loc[self.raw_rpf['name'].isin(list(self.gene)), ]
        self.sample_num = int((len(rpf_mer.columns) - 6) / 3)
        self.sample_name = pd.Series(rpf_mer.columns[6::]).str[:-3].drop_duplicates().tolist()
        self.sample_dict = OrderedDict()
        rpf_idx = rpf_mer.iloc[:, 0:6].copy()

        for now_sp in self.sample_name:
            print('Import the RPFs file {file_name}.'.format(file_name=now_sp), flush=True)
            now_sp_index = [now_sp + '_f0', now_sp + '_f1', now_sp + '_f2']
            now_sp_rpf = rpf_mer.loc[:, now_sp_index].copy()
            now_rpf = pd.concat([rpf_idx, now_sp_rpf], axis=1)

            # get the high expression gene id
            now_rpf["sum"] = pd.DataFrame(now_sp_rpf.sum(axis=1))
            cds_rpf = now_rpf[now_rpf["region"] == "cds"][["name", "sum"]].groupby("name")["sum"].apply(np.sum)
            self.high_gene = cds_rpf[cds_rpf > self.rpf_num]

            # filter the high expression genes
            if self.norm:
                total_rpf = now_sp_rpf.sum().sum()
                now_sp_rpm = now_sp_rpf.div(total_rpf) * 1e7
                now_rpm = pd.concat([rpf_idx, now_sp_rpm], axis=1)
                self.high_rpf = now_rpm[now_rpm["name"].isin(self.high_gene.index)]
            else:
                self.high_rpf = now_rpf[now_rpf["name"].isin(self.high_gene.index)]

            tis_rpf = self.high_rpf[self.high_rpf["from_tis"].isin(range(-self.utr5, self.cds))]
            tts_rpf = self.high_rpf[self.high_rpf["from_tts"].isin(range(-self.cds + 1, self.utr3 + 1))]
            self.tis_meta = tis_rpf.groupby("from_tis")[now_sp_index].apply(sum).div(len(self.high_gene)).reset_index()
            self.tts_meta = tts_rpf.groupby("from_tts")[now_sp_index].apply(sum).div(len(self.high_gene)).reset_index()

            self.tis_meta.rename(columns={"from_tis": "Codon"}, inplace=True)
            self.tts_meta.rename(columns={"from_tts": "Codon"}, inplace=True)

            self.output_meta(now_sp)
            self.sample_dict[now_sp] = [self.tis_meta, self.tts_meta, len(self.high_gene), now_sp_index]

            # merge all tis and tts meta
            if self.merge_meta is None:
                self.merge_meta = pd.concat([self.tis_meta, self.tts_meta], axis=0)
            else:
                self.merge_meta = pd.concat([self.merge_meta, self.tis_meta, self.tts_meta], axis=0)


    def draw_bar_metaplot(self):
        for sample_name, mess in self.sample_dict.items():
            tis_meta, tts_meta, gene_num, sample_frame = mess[0], mess[1], mess[2], mess[3]
            
            if all(value == 0 for value in tis_meta['Density']):
                print("Warning: All Density values are 0.")

            out_pdf = sample_name + "_meta_bar_plot.pdf"
            out_png = sample_name + "_meta_bar_plot.png"

            matplotlib.use('AGG')
            colors = {0: '#66c2a5', 1: '#fc8d62', 2: '#8da0cb'}

            fig = plt.figure(figsize=(15, 5), dpi=300)
            # draw the figure around the TIS region
            ax1 = fig.add_subplot(121)
            for key, group in tis_meta.groupby('Frame'):
                ax1.bar(group['Nucleotide'], group['Density'], color=colors[key], width=0.6, label=f'Frame {key}')

            # Add some text for labels, title and custom x-axis tick labels, etc.
            ax1.set_title("Mean RPFs of TIS region ({number} genes)".format(number=gene_num), fontsize=14)
            ax1.set_ylabel('Mean RPFs', fontsize=12)
            ax1.set_xlabel('From start codon (nt)', fontsize=12)
            ax1.set_xticks([-self.utr5 * 3, 0, self.cds * 3 - 1])
            # ax1.set_xticklabels([-utr5 * 3, 0, cds * 3], fontsize=20)
            plt.xticks(fontsize=12)
            plt.yticks(fontsize=12)
            ax1.legend(fontsize=12)

            # draw the figure around the tts region
            ax2 = fig.add_subplot(122)
            for key, group in tts_meta.groupby('Frame'):
                ax2.bar(group['Nucleotide'], group['Density'], color=colors[key], width=0.6, label=f'Frame {key}')

            # Add some text for labels, title and custom x-axis tick labels, etc.
            ax2.set_title("Mean RPFs of TTS region ({number} genes)".format(number=gene_num), fontsize=14)
            ax2.set_ylabel('Mean RPFs', fontsize=12)
            ax2.set_xlabel('From stop codon (nt)', fontsize=12)
            ax2.set_xticks([-self.cds * 3 + 1, 0, self.utr3 * 3])
            # ax2.set_xticklabels([-cds * 3 + 1, 0, utr3 * 3], fontsize=20)
            plt.xticks(fontsize=12)
            plt.yticks(fontsize=12)
            ax2.legend(fontsize=12)

            fig.tight_layout()
            # plt.show()
            fig.savefig(fname=out_pdf)
            fig.savefig(fname=out_png)
            plt.close()

    def draw_line_metaplot(self):
        for sample_name, mess in self.sample_dict.items():
            tis_meta, tts_meta, gene_num, sample_frame = mess[0], mess[1], mess[2], mess[3]

            out_pdf = sample_name + "_meta_line_plot.pdf"
            out_png = sample_name + "_meta_line_plot.png"

            matplotlib.use('AGG')
            fig = plt.figure(figsize=(15, 5), dpi=300)
            # draw the figure around the TIS region
            ax1 = fig.add_subplot(121)
            tis_x = tis_meta['Nucleotide']
            ax1.plot(tis_x, tis_meta["Density"])
            # Add some text for labels, title and custom x-axis tick labels, etc.
            ax1.set_title("Mean RPFs of TIS region ({number} genes)".format(number=gene_num), fontsize=14)
            ax1.set_ylabel('Mean RPFs', fontsize=12)
            ax1.set_xlabel('from start codon (nt)', fontsize=12)
            ax1.set_xticks([-self.utr5 * 3, 0, self.cds * 3 - 1])
            # ax1.set_xticklabels([-utr5 * 3, 0, cds * 3], fontsize=20)
            plt.xticks(fontsize=12)
            plt.yticks(fontsize=12)

            # draw the figure around the tts region
            ax2 = fig.add_subplot(122)
            tts_x = tts_meta['Nucleotide']
            ax2.plot(tts_x, tts_meta["Density"])
            # Add some text for labels, title and custom x-axis tick labels, etc.
            ax2.set_title("Mean RPFs of TTS region ({number} genes)".format(number=gene_num), fontsize=14)
            ax2.set_ylabel('Mean RPFs', fontsize=12)
            ax2.set_xlabel('from stop codon (nt)', fontsize=12)
            ax2.set_xticks([-self.cds * 3 + 1, 0, self.utr3 * 3])
            # ax2.set_xticklabels([-cds * 3 + 1, 0, utr3 * 3], fontsize=20)
            plt.xticks(fontsize=12)
            plt.yticks(fontsize=12)

            fig.tight_layout()
            # plt.show()
            fig.savefig(fname=out_pdf)
            fig.savefig(fname=out_png)
            plt.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : CDT.py


import sys
import matplotlib
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import seaborn as sns
from . import RPFs


class CodonDecodingTime(object):

    def __init__(self, args):
        self.mrna_file = args.list
        self.gene_table = None

        self.rpf_file = args.rpf
        self.rna_file = args.rna

        self.sample_num = 0
        self.sample_name = None
        self.rel_name = None
        self.merged_rpf = None
        self.merged_rpf_na = None

        # for the high expression gene filter
        # self.norm = args.normal
        self.min = args.min

        # rpf table
        self.rpf_sample = None
        self.total_rpf_num = None
        self.rpf_gene_sum = None
        self.rpf_gene = None
        self.high_rpf = None

        # rna table
        self.rna_sample = None
        self.total_rna_num = None
        self.rna_gene_sum = None
        self.rna_gene = None
        self.high_rna = None

        # filter the stable codon coverage range
        self.scale = args.scale
        self.site = args.site
        self.frame = args.frame
        self.tis = args.tis
        self.tts = args.tts

        # table for cdt
        self.rpf_norm_dst = None
        self.cdt = None

        self.stop = args.stop
        self.codon_dict, self.codon_table = RPFs.codon_table()

        if self.stop:
            # remove the TAA TAG TGA from the codon table
            del self.codon_dict['TAA']
            del self.codon_dict['TAG']
            del self.codon_dict['TGA']

        self.output = args.output


    def import_gene(self):
        '''
        @Message  : import the gene file
        @Input    : self.gene --> gene file
        @Return   : self.gene --> gene dataframe
        @Flow     : step1 --> import the gene file
        '''

        self.gene_table = pd.read_csv(self.mrna_file, sep='\t', index_col=False)
        self.gene_table['length'] = self.gene_table['utr5_length'] + self.gene_table['cds_length'] + self.gene_table['utr3_length']
        self.gene_table.set_index('transcript_id', inplace=True)
        self.gene_table.index.name = 'name'


    def import_density(self):
        """
        import the Ribo-seq and RNA-seq density file.
        By default, the two data names are different.

        return the main results as follows:
        samples name,
        high expression gene density
        high expression gene name
        total rpf/reads
        """

        # import the rpf density
        rpf_results = RPFs.import_rpf(rpf_file=self.rpf_file, sample_num=None,
                                      sample_name=None, sites=self.site, frame=self.frame,
                                      gene=self.mrna_file, rpf_num=self.min, tis=self.tis, tts=self.tts)
        self.rpf_sample = rpf_results[1]
        self.total_rpf_num = rpf_results[4]
        self.rpf_gene_sum = rpf_results[5]
        self.rpf_gene = rpf_results[6]
        self.high_rpf = rpf_results[7].copy()

        del rpf_results

        print('', flush=True)

        # import the rna density
        rna_results = RPFs.import_rpf(rpf_file=self.rna_file, sample_num=None,
                                      sample_name=None, sites=self.site, frame=self.frame,
                                      gene=self.mrna_file, rpf_num=self.min, tis=self.tis, tts=self.tts)
        self.rna_sample = rna_results[1]
        self.total_rna_num = rna_results[4]
        self.rna_gene_sum = rna_results[5]
        self.rna_gene = rna_results[6]
        self.high_rna = rna_results[7].copy()

        del rna_results


    def merge_rpf_rna(self):
        """
        get the overlap gene from rpf and mrna density file
        only these genes will submit to follow analysis
        """
        
        self.overlap_gene = set(self.rpf_gene) & set(self.rna_gene)

        self.high_rpf = self.high_rpf.loc[self.high_rpf['name'].isin(self.overlap_gene)]
        self.high_rna = self.high_rna.loc[self.high_rna['name'].isin(self.overlap_gene)]


    def normalize_density(self):
        """
        normalize the density to rpkm. rpkm = (10^9 * C) / (N * L)
        N: total reads number, L: gene length, C: reads number of the gene
        """
        # filter the rpf_gene_sum index with the self.overlap_gene
        gene_len = self.gene_table.loc[list(self.overlap_gene),'length']

        self.rpf_gene_sum = self.rpf_gene_sum.loc[list(self.overlap_gene)]
        self.rpf_gene_rpm = 1e6 * self.rpf_gene_sum / self.total_rpf_num
        self.rpf_gene_rpkm = 1e9 * self.rpf_gene_sum.div(gene_len, axis=0) / self.total_rpf_num

        self.rna_gene_sum = self.rna_gene_sum.loc[list(self.overlap_gene)]
        self.rna_gene_rpm = 1e6 * self.rna_gene_sum / self.total_rna_num
        self.rna_gene_rpkm = 1e9 * self.rna_gene_sum.div(gene_len, axis=0) / self.total_rna_num


    @staticmethod
    def scale_method(scale, cdt):
        if scale == 'minmax':
            # relative_cdt = (cdt - cdt.min()) / (cdt.max() - cdt.min())
            relative_cdt = cdt / cdt.max()
        elif scale == 'zscore':
            relative_cdt = (cdt - cdt.mean()) / cdt.std()
        else:
            print('Unknown scale method.', flush=True)
            sys.exit()
        return relative_cdt
    

    def calc_rpf_norm(self):
        '''
        @Message  : function for rpf normalization.
        @Input    : self.high_rpf --> high expression gene rpf density
        @Return   : self.rpf_norm_dst --> normalized rpf density with RNA-seq RPKM
        @Flow     : step1 --> calculate the average density and normalized rpf density of each codon
                    step2 --> merge average density 
        '''
        
        # calculate the average density of each codon
        self.high_rpf = self.high_rpf[self.high_rpf['region'] == 'cds']
        high_rpf_group = self.high_rpf.groupby('name')
        now_num = 0
        rpf_norm_list = []

        self.count_column = [i + '_count' for i in self.rpf_sample]
        self.norm_column = [i + '_norm' for i in self.rpf_sample]

        for gene, rpf in high_rpf_group:
            # counter
            now_num += 1
            if now_num % 500 == 0:
                print('Row: {rows}, {gene}.'.format(rows=now_num, gene=gene), flush=True)
            if now_num == len(high_rpf_group):
                print('Row: {rows}, {gene}.'.format(rows=now_num, gene=gene), flush=True)

            # sum the rpf and each codon number
            rpf_sum = rpf.groupby('codon')[self.rpf_sample].sum()
            rpf_norm = rpf_sum.div(self.rna_gene_rpkm.loc[gene].to_list(), axis=1)

            rpf[self.rpf_sample] = rpf[self.rpf_sample].map(lambda x: 1 if x != 0 else 0)
            codon_sum = rpf.groupby('codon')[self.rpf_sample].sum()

            rpf_sum.columns = self.count_column
            rpf_norm.columns = self.norm_column

            temp_rpf_norm = pd.concat([codon_sum, rpf_sum, rpf_norm], axis=1)
            rpf_norm_list.append(temp_rpf_norm)

        self.rpf_norm_dst = pd.concat(rpf_norm_list, axis=0)
                
        # check the stop codon
        if self.stop:
            # if cdt table contains the TAA TAG TGA, remove them
            self.rpf_norm_dst = self.rpf_norm_dst.drop(['TAA', 'TAG', 'TGA'], errors='ignore')

        # drop the na and inf values
        self.rpf_norm_dst.replace([np.inf, -np.inf], np.nan, inplace=True)

        self.rpf_norm_dst.reset_index('codon', inplace=True)

    def codon_decoding_time(self):
        '''
        @Message  : function for codon decoding time calculatioin.
        @Input    : param --> description
        @Return   : output --> description
        @Flow     : step1 --> merge all norm
                    step2 --> calculate cdt
                    step3 --> calculate relative cdt
                    step4 --> output the cdt table
        '''

        # merge average density and calculate cdt
        sum_rpf_norm = self.rpf_norm_dst.groupby('codon').sum()

        # calculate the absolute cdt
        cdt = sum_rpf_norm[self.count_column] / sum_rpf_norm[self.rpf_sample].rename(columns=dict(zip(self.rpf_sample, self.count_column)))
        norm_cdt = sum_rpf_norm[self.norm_column] / sum_rpf_norm[self.rpf_sample].rename(columns=dict(zip(self.rpf_sample, self.norm_column)))

        cdt.columns = [i + '_absolute_cdt' for i in self.rpf_sample]
        norm_cdt.columns = [i + '_norm_cdt' for i in self.rpf_sample]

        # calculate the relative cdt
        relative_cdt = self.scale_method(self.scale, norm_cdt)
        relative_norm_cdt = self.scale_method(self.scale, relative_cdt)

        relative_cdt.columns = [i + '_relative_cdt' for i in self.rpf_sample]
        relative_norm_cdt.columns = [i + '_norm_relative_cdt' for i in self.rpf_sample]

        # merge all the cdt table
        self.codon_table = pd.DataFrame(self.codon_dict).T
        self.codon_table.columns = ['AA', 'Abbr']
        self.codon_table = pd.concat([self.codon_table, sum_rpf_norm, cdt, relative_cdt, norm_cdt, relative_norm_cdt], axis=1)
        self.codon_table.sort_values('Abbr', inplace=True)
        self.codon_table.index.name = 'Codon'
        self.codon_table.reset_index(inplace=True)

        self.codon_table.to_csv(self.output + '_cdt.txt', header=True, index=False, sep='\t')


    def draw_cdt_corr(self):
        out_pdf = self.output + "_cdt_corrplot.pdf"
        out_png = self.output + "_cdt_corrplot.png"

        occ_corr = self.codon_table.filter(regex='_norm_cdt').astype('float').corr()
        occ_corr.to_csv(self.output + '_cdt_corr.txt', sep='\t', index=True)

        matplotlib.use('AGG')
        fig = plt.figure(figsize=(6 + 0.2 * len(self.rpf_sample),
                                  8 + 0.2 * len(self.rpf_sample)),
                                  dpi=300)
        sns.heatmap(data=occ_corr, 
                    annot=True, 
                    fmt=".4f", 
                    cmap="vlag",
                    linewidths=.01)
        plt.tight_layout()
        # plt.show()

        plt.savefig(fname=out_pdf)
        plt.savefig(fname=out_png)
        plt.close()

    def draw_cdt_heat(self):
        # output the absolute occupancy heatmap
        out_pdf = self.output + "_cdt_heatplot.pdf"
        out_png = self.output + "_cdt_heatplot.png"

        matplotlib.use('AGG')
        fig = plt.figure(figsize=(10 + 0.5 * len(self.rpf_sample), 20), dpi=300)
        sns.heatmap(data=self.codon_table.filter(regex='_norm_cdt').astype('float'),
                    yticklabels=self.codon_table['Codon'] + '[' + self.codon_table['Abbr'].astype(str) + ']',
                    annot=True, 
                    fmt=".4f", 
                    cmap="vlag", 
                    linewidths=0)
        plt.tight_layout()
        # plt.show()

        plt.savefig(fname=out_pdf)
        plt.savefig(fname=out_png)
        plt.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Script  : getTissueFreqSeq.py
# @Time    : 2020/3/24 11:18
# @project : py-scripts
# @Version : python 3.7
# @Author  : Rensc
# @E-mail  : rensc0718@163.com


import argparse
from argparse import RawTextHelpFormatter
import gc
import time
import copy


# get arguments and define the scripts usage
def parse_args():
    parser = argparse.ArgumentParser(description='This script is used to combine the tissue freq.',
                                     formatter_class=RawTextHelpFormatter)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s 1.0')

    required = parser.add_argument_group(title='Required parameters')
    required.add_argument('-i', dest='input', required=True, type=str,
                          help='input file contain path and name of each sample')
    required.add_argument('-b', dest='barcode', required=False, type=str, default='all_barcodes.txt',
                          help='output file contain reads count of each sample (default: %(default)s)')
    required.add_argument('-o', dest='output', required=False, type=str, default='sRNA_tissue_freq_seq.fa',
                          help='output file contain non-redundant reads of each sample (default: %(default)s)')
    required.add_argument('-t', dest='txt', required=False, default=False, action='store_true',
                            help='output in the table format. (default: %(default)s)')
    args = parser.parse_args()
    return args


# global parameter
sample_list = {}
barcode = []
srna = {}


# read the list file
def list_read(file_list):
    print('Reading file list.')
    with open(file_list, 'r') as ids_input:
        num = 0
        for line in ids_input:
            pre_ids = line.strip().split('/')[-1].split('.')[0]
            flag = [(pre_ids == v.split('\t')[0]) for k, v in sample_list.items()]
            if True in flag:
                print('There is a repeats sample in your list:\n%s ' % pre_ids, flush=True)
                exit(1)
            else:
                sample_list[num] = pre_ids + '\t' + line.strip()
            barcode.append([pre_ids, 0])
            num += 1


# read srna file in fa format
def srna_read():
    total = len(sample_list.keys()) + 1
    temp_list = [0 for i in range(total)]
    
    for num, file in sample_list.items():
        names = file.split('\t')[1]
        print('Reading: %s' % names, flush=True)
        
        with open(names, 'r') as srna_input:
            for seq in srna_input:
                if seq.startswith('>'):
                    pass
                else:
                    seq = seq.strip()
                    if srna.get(seq):
                        srna[seq][num] += 1
                        barcode[num][1] += 1
                    else:
                        srna[seq] = copy.copy(temp_list)
                        srna[seq][num] += 1
                        barcode[num][1] += 1


# sum barcode
def sum_barcode():
    print('Sum total barcode.')
    for seq, num in srna.items():
        srna[seq][-1] = sum(num)


# output nn-redundant srna reads
def srna_write(output_file):
    print('Writing: %s' % output_file, flush=True)
    with open(output_file, 'w') as srna_output:
        num = 0
        for seq, counts in srna.items():
            num += 1
            counts = map(str, counts)
            srna_output.writelines('>t' + str(num) + ' ' + ' '.join(counts) + '\n' + seq.lower() + '\n')


# output nn-redundant srna reads
def barcode_write(barcode_file):
    print('Writing: %s' % barcode_file, flush=True)
    with open(barcode_file, 'w') as barcode_output:
        sum_total = 0
        for line in barcode:
            sum_total += line[1]
            barcode_output.writelines(line[0] + '\t' + str(line[1]) + '\n')
        barcode_output.writelines('total\t' + str(sum_total) + '\n')


# get time
def now_time():
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), flush=True)


# main programme
def main():
    now_time()
    args = parse_args()
    list_read(args.i)
    srna_read()
    sum_barcode()
    srna_write(args.o)
    barcode_write(args.b)
    gc.collect()
    now_time()


# run programme here
if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project : riboParser
# @Script  : make_ribo_ref.py


from .ribo import ArgsParser
from .ribo.GenePred import *


def main():
    ArgsParser.now_time()
    print('\nMake the reference for riboParser.', flush=True)
    print('Step1: Checking the input Arguments.', flush=True)
    args = ArgsParser.make_ribo_ref()
    ribo_ref = GenePred(args)

    print('\nStep2: Import genome sequence.', flush=True)
    ribo_ref.read_genome()

    print('\nStep3: Format gtf annotation.', flush=True)
    ribo_ref.gtf2gp()
    ribo_ref.read_genepred()
    ribo_ref.get_rep_transcript()
    ribo_ref.add_utr()

    print('\nStep4: Output gtf annotation.', flush=True)
    ribo_ref.write_txt()
    ribo_ref.gp2gtf()

    print('\nStep5: Retrieve mRNA sequence.', flush=True)
    ribo_ref.get_seq()
    ribo_ref.write_seq()

    print('\nAll done.', flush=True)


if __name__ == '__main__':
    main()
